(function($) {

    // attach to jQuery
    $.extend({
        ContextHelp : function(options)
        {
            const DEFAULT_CONTEXTHELP_KEY = "GLOBAL_CONTEXTHELP_NOT_DEFINED";
            const CONTEXT_HELP_TRANSLATION_SUFFIX = "_CONTEXT_HELP";
            const CONTEXT_HELP_CONTENT_SELECTOR = ".contextHelp-content";
            
            var defaults = {
                listeningSelector: "input[type='text'], input[type='checkbox'], input[type='password'], input[type='radio'], textarea, select, [data-custom-contexthelp-key]",
                translateMode:false
            };
            
            options = $.extend(
                {},
                defaults,
                options
            );

            function _getContextHelper(options)
            {
                var _helper = _initContextHelp(options);
                return _helper;
            }
    
            function _initContextHelp(options)
            {
                function collectListeningElements(listeningSelector)
                {
                    return $(listeningSelector); 
                };
                
                return {
                    listeningElements:collectListeningElements(options.listeningSelector),
                    initializeTranslations:initializeTranslations,
                    initializeOnFocusEvents:initializeOnFocusEvents,
                    initializeOnMouseOverEvents:initializeOnMouseOverEvents,
                    translations:{},
                    addlisteningElement:addlisteningElement,
                    triggerContextHelp:triggerElement,
                    afterTranslate: function() {}
                };
            }
            
            var initializeTranslations = function()
            {
                var translationKeys = {};
                this.listeningElements.each(function(i, element)
                {
                    if (typeof $(element).data("custom-contexthelp-key") === "undefined") {
                        translationKeys[i] = element.id.toUpperCase() + CONTEXT_HELP_TRANSLATION_SUFFIX;
                    } else {
                        translationKeys[i] = $(element).data("custom-contexthelp-key");
                    }
                });
                
                translationKeys[translationKeys.length] = DEFAULT_CONTEXTHELP_KEY;
                translate(translationKeys);
            };

            function translate(translationKeys)
            {
                if (typeof translationKeys === 'string') {
                    translationKeys = {
                        0:translationKeys
                    }
                }
                $.ajax({
                    url:"/index.php?r=site/contextHelp",
                    data:{
                        translationKeys:translationKeys
                    },
                    dataType:"JSON",
                    type:"POST",
                    success:function(data) {
                        // contextHelper is a global variable defined in layouts/main.php
                        $.extend(contextHelper.translations, data);
                        contextHelper.afterTranslate();
                    },
                    error: function(e){
                        console.log(e.responseText);
                    }
                });
            }
            
            var initializeOnFocusEvents = function ()
            {
                $("html").on("focus", this.listeningElements.selector,function()
                {
                    triggerContextHelp(this);

                });
            };
            
            var initializeOnMouseOverEvents = function ()
            {
                $("html").on("mouseover", this.listeningElements.selector,function()
                {
                    triggerContextHelp(this);
                });
            };
            
            
            function triggerContextHelp(element)
            {
                var translationKey = $(element).data("custom-contexthelp-key");
                if (typeof translationKey === "undefined") {
                    var translationKey = element.id.toUpperCase() + CONTEXT_HELP_TRANSLATION_SUFFIX;
                }

                var translatedKey = contextHelper.translations[translationKey];

                $(CONTEXT_HELP_CONTENT_SELECTOR).text("");

                if (options.translateMode === true) {
                    insertTranslationTags(element, translationKey, translatedKey);
                } else {
                    inserttranslation(translationKey, translatedKey);
                }
                // open external links in new window:
                $(CONTEXT_HELP_CONTENT_SELECTOR + " a").filter(function() {
                    return this.hostname != window.location.hostname;
                }).attr('target', '_blank');
                $(".contextHelp-container").show();
            }

            var triggerElement = function (elementSelector) {
                var element = $(elementSelector);
                triggerContextHelp(element[0]);
            }

            function insertTranslationTags(element, translationKey, translatedKey)
            {
                if ($(element).data("custom-contexthelp-key"))
                    {
                        translationKey = $(element).data("custom-contexthelp-key");

                        if (typeof contextHelper.translations[translationKey] === "undefined") {
                            contextHelper.translations[translationKey] = translationKey;
                            translatedKey = translationKey;
                        } else {
                            translatedKey = contextHelper.translations[translationKey];
                        }
                    }
                    var translationTag  = $("<i />").attr("data-translation-id", translationKey).append(translatedKey);
                    $(CONTEXT_HELP_CONTENT_SELECTOR).append(translationTag);
                    $(document).trigger("ajaxSuccess"); // trigger translator update function from layouts/main.php
            }
            
            function inserttranslation(translationKey, translatedKey) {
                if ("<p>" + translationKey + "</p>" !== translatedKey) {
                    $(CONTEXT_HELP_CONTENT_SELECTOR).append(translatedKey);
                } else {
                    translatedKey = contextHelper.translations[DEFAULT_CONTEXTHELP_KEY];
                    $(CONTEXT_HELP_CONTENT_SELECTOR).append(translatedKey);
                }
            }

            var addlisteningElement = function (selector, translationKey)
            {
                var newElement = $(selector);

                if (typeof translationKey !== 'undefined' && translationKey !== '') {
                    newElement.data("custom-contexthelp-key", translationKey);
                } else {
                    newElement.data("custom-contexthelp-key", newElement.attr('id').toUpperCase() + CONTEXT_HELP_TRANSLATION_SUFFIX);
                }

                translate(newElement.data("custom-contexthelp-key"));

                newElement.on("mouseover", this.listeningElements.selector,function()
                {
                    triggerContextHelp(this);
                });

                newElement.on("focus", this.listeningElements.selector,function()
                {
                    triggerContextHelp(this);
                });
            };
            
            var _contextHelp = _getContextHelper(options);
            _contextHelp.initializeTranslations();
            _contextHelp.initializeOnFocusEvents();
            _contextHelp.initializeOnMouseOverEvents();
            
            $('.modal').on("shown", function () {
                focusFirstElement();
            });
            
            $('.modal-body').on("scroll", function () {
                var currScrollOffet = $(this).scrollTop();
                $(".contextHelp-container:visible").stop();
                $(".contextHelp-container:visible").animate({
                    "margin-top": currScrollOffet
                }, "fast");
            });

            $("[data-toggle='tab']").click(function() {
                focusFirstElement();
            });
            
            function focusFirstElement() {
                var selector = ".tab-content input[type='text']";
                selector = selector + ", .tab-content input[type='checkbox']";
                selector = selector + ", .tab-content input[type='password']";
                selector = selector + ", .tab-content textarea";
                selector = selector + ", .tab-content select";
                
                var firstElement = $(selector).filter(":visible:enabled:first");
                firstElement.focus();
            }
            
            return _contextHelp;
        }
        
    });
})(jQuery);