<?php

class ExportHelper
{

    public static $specificAttributeArray = array();

    public static function exportSpecificModelAsCSV($modelName, $specificAttributeArray)
    {
        ExportHelper::setSpecificAttributeArray($specificAttributeArray);
        ExportHelper::exportModelAsCSV($modelName);
    }

    private static function setSpecificAttributeArray($specificFindAttribute)
    {
        ExportHelper::$specificAttributeArray = $specificFindAttribute;
    }

    public static function exportModelAsCSV($modelName)
    {
        $attributes = ExportHelper::getExportRelevantModelAttributes($modelName);
        ExportHelper::exportAsCsv($modelName, array_keys($attributes));
    }

    public static function exportAsCsv($modelName, $attributes = array())
    {
        $limit = 1;
        $offset = 0;
        ExportHelper::setCsvHeaders($modelName);
        while ($models = ExportHelper::getModelsForExport($modelName, $limit, $offset)) {
            $csvData = array();
            if ($offset === 0) {
                ExportHelper::outputCsvHeader($attributes, $csvData);
            }
            foreach ($models as $model) {
                ExportHelper::createCsvDataFromModel($model, $csvData, $attributes);
            }
            if (!empty($csvData)) {
                ExportHelper::outputCsvData($csvData);
            }
            $offset = $offset + $limit;
        }
    }

    private static function setCsvHeaders($modelName)
    {
        $fileName = "phonalisa_" . lcfirst($modelName) . "_" . date('Y-m-d_G-i-s') . ".csv";
        header('Content-type: text/csv');
        header("Content-disposition: attachment; filename=$fileName");
        header("Pragma: no-cache");
        header("Expires: 0");
    }

    private static function getModelsForExport($modelName, $limit, $offset)
    {
        $findAttributes = ExportHelper::$specificAttributeArray;
        $models = PhonalisaRecord::model($modelName)->find($findAttributes, "id", $limit, $offset);

        if (empty($models)) {
            return false;
        }
        return $models;
    }

    private static function outputCsvHeader($attributes, &$csvData)
    {
        if (!$attributes) {
            return;
        }
        $csvData[] = $attributes;
    }

    private static function createCsvDataFromModel($model, &$csvData, $attributes = array())
    {
        if (empty($attributes)) {
            $attributes = array_keys($model->attributes);
        }
        $modelData = array();
        foreach ($attributes as $attribute) {
            $modelData[$attribute] = $model->$attribute;
        }
        $csvData[] = $modelData;
    }

    private static function outputCsvData($csvData, $delimiter = ";")
    {
        foreach ($csvData as $csvLineData) {
            echo implode($delimiter, $csvLineData) . "\n";
        }
    }

    public static function getExportRelevantModelAttributes($modelName)
    {
        $attributes = $modelName::model()->getAttributes();
        if (method_exists($modelName, 'getExportNotRelevantAttributes')) {
            foreach ($modelName::model()->getExportNotRelevantAttributes() as $notRelevantKey) {
                unset($attributes[$notRelevantKey]);
            }
        }
        return $attributes;
    }
}
