<?php

class CalendarType
{

    const personal = 1;
    const group = 2;

    public static function toString($type)
    {
        if ($type == CalendarType::personal) {

            $type = __('CALENDAR_MODEL_TYPE_PRIVATE');
        } else if ($type == CalendarType::group) {

            $type = __('CALENDAR_MODEL_TYPE_GROUP');
        }
        return $type;
    }
}

class Calendar extends PhonalisaRecord
{
    public $id;
    public $owner;
    public $title;
    public $type;
    public $color;    

    public static function myAttributes()
    {
        return array(
            'id' => 'id',
            'owner' => 'owner',
            'title' => 'title',
            'type' => 'type',
            'color' => 'color',
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Calendar the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'calendar';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('type, title', 'required'),
            array('type', 'numerical', 'integerOnly' => true),
            array('title, color, owner', 'safe'),
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('id, owner, title, type', 'safe', 'on' => 'search'),
        );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => __('ID'),
            'title' => __('CALENDAR_MODEL_TITLE'),
            'type' => __('CALENDAR_MODEL_TYPE'),
            'color' => __('CALENDAR_MODEL_COLOR'),
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search($order = 'title')
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria;
        $filter = $this->getFilter($criteria, $order);

        $filter['pagination'] = array(
            'pageSize' => getGridPageSize('calendar-grid'),
        );

        if ($filter != null) {
            return new CPhonalisaDataProvider($this, $filter);
        }
    }

    public function renderPermissions(PhonalisaRecord $data, $row, $column)
    {
        $htmlOptions = array(
            'class' => $column->id
        );
        $permission = NULL;
        $hiddenValue = '0';

        $parentGroups = $data->getParents(true);
        foreach ($parentGroups as $parent) {
            $permission = Permissions::model()->getPermissionsArray(array('type' => $column->id, 'groupId' => $parent->id, 'itemId' => $this->id));

            if ($permission) {
                $htmlOptions['disabled'] = 'disabled';
                $htmlOptions['data-parent'] = 'true';
                $checked = true;
                break;
            }
        }
        if (!count($permission)) {

            $permission = Permissions::model()->getPermissionsArray(array('type' => $column->id, 'groupId' => $data->id, 'itemId' => $this->id));
            if ($permission) {
                $checked = true;
            } else {
                $checked = false;
            }
        }

        if (!array_key_exists('disabled', $htmlOptions)) {
            $hiddenValue = '0';
        }

        $html = CHtml::hiddenField('Permissions[' . $data->id . '][' . $column->id . ']', $hiddenValue);
        $html.= CHtml::checkBox('Permissions[' . $data->id . '][' . $column->id . ']', $checked, $htmlOptions);

        return $html;
    }

    public function renderCalenderType(PhonalisaRecord $data, $row, $column)
    {
        return CalendarType::toString($data->type);
    }

    public function isDarkColor()
    {
        $hexcolor = str_replace('#', '', $this->color);

        $r = hexdec(substr($hexcolor, 0, 2));
        $g = hexdec(substr($hexcolor, 2, 2));
        $b = hexdec(substr($hexcolor, 4, 2));
        $yiq = (($r * 299) + ($g * 587) + ($b * 114)) / 1000;
        return !($yiq >= 128);
    }

    public function getCalendarList($onlyWritable = false)
    {
        $userModel = User::model()->findByPk(user()->sudoUser);
        if ($userModel) {
            $models = $userModel->getCalendars($onlyWritable ? 'write_calendar' : 'read_calendar');
            $list = array();

            foreach ($models as $calendar) {
                $list[$calendar->id] = $calendar->title;
            }

            $lowercasList = array();
            foreach ($list as $key => $value) {
                $lowercasList[$key] = strtolower($value);
            }

            asort($lowercasList);
            foreach ($lowercasList as $key => $value) {

                $lowercasList [$key] = $list[$key];
            }
            if (!empty($lowercasList)) {
                return $lowercasList;
            } else {
                return array('' => __('CALENDAR_MODEL_NO_CALENDAR_AVAILABLE'));
            }
        } else {
            app()->user->logout();
            app()->user->loginRequired();
        }
    }

    public function getColoredCalendarList($onlyWritable = false)
    {
        $userModel = User::model()->findByPk(user()->sudoUser);

        if (!$userModel) {
            return false;
        }

        $models = $userModel->getCalendars($onlyWritable ? 'write_calendar' : 'read_calendar');

        $list = array();

        foreach ($models as $calendar) {
            $list[$calendar->id] = $calendar->title;
        }

        uasort($list, function($a, $b) {
            return strnatcmp(strtolower($a), strtolower($b));
        });

        foreach ($list as $id => $title) {
            $list[$id] = $models[$id]->getColoredTitle();
        }
        return $list;
    }

    public function getColoredTitle()
    {
        return $this->title . '|||' . $this->color;
    }

    public function getEventTypeList()
    {
        $models = EventType::model()->findAll();
        $list = array();

        foreach ($models as $eventType) {
            $list[$eventType->id] = $eventType->name;
        }
        $lowercasList = array();
        foreach ($list as $key => $value) {
            $lowercasList[$key] = strtolower($value);
        }

        asort($lowercasList);
        foreach ($lowercasList as $key => $value) {

            $lowercasList [$key] = $list[$key];
        }
        return $lowercasList;
    }

    public function getEventSources()
    {
        $models = $this->findAll();
        $sources = array();

        $userModel = User::model()->findByPk(user()->sudoUser);

        $readableClaendars = $userModel->getCalendars('read_calendar');
        $writeableClaendars = $userModel->getCalendars('write_calendar');

        foreach ($readableClaendars as $model) {
            $sourceOptions = array();
            $sourceOptions['id'] = $model->id;
            $sourceOptions['title'] = $model->title;
            $sourceOptions['color'] = $model->color;
            $sourceOptions['url'] = Yii::app()->createUrl('calendarGeneral/default/getJsonEvents', array('id' => $model->id));
            $sourceOptions['editable'] = array_key_exists($model->id, $writeableClaendars);
            $sources[$model->id] = $sourceOptions;
        }
        if (empty($sources)) {
            //FIXME  ?
            throw new CHttpException(403, 'Forbidden');
        }
        return $sources;
    }

    public function beforeDelete()
    {
        $permissions = Permissions::model()->findAllByAttributes(array('itemId' => $this->id));
        while (count($permissions)) {
            foreach ($permissions as $permission) {
                $permission->delete();
            }
            $permissions = Permissions::model()->findByAttributes(array('itemId' => $this->id));
        }
        $permissionModel = new Permissions();
        $permissionModel->createPermissionsFile();
        $this->deleteICloudContactsList();
        $this->deleteCalendarEvents();
        return parent::beforeDelete();
    }

    private function deleteICloudContactsList()
    {
        $iCloudContactsList = iCloudContactsList::model()->findAllByAttributes(array('plDirectoryId' => $this->id));
        foreach ($iCloudContactsList as $list) {
            $list->delete();
        }
    }

    protected function deleteCalendarEvents()
    {
        $events = CalendarEvent::model()->findAllByAttributes(array('calendar' => $this->id));
        foreach($events as $event) {
            $event->delete();
        }
    }

    public function allCalendarProvider()
    {
        return new CArrayDataProvider($this->findAll(), array('pagination' => false));
    }

    public function humanReadableSearchFilterForType()
    {
        $list = array(
            CalendarType::personal => CalendarType::toString(CalendarType::personal),
            CalendarType::group => CalendarType::toString(CalendarType::group),
        );
        asort($list);
        return $list;
    }
}
