<?php

class UserSettings extends PhonalisaRecord
{

    public $userId;
    public $ringVolume = 5;
    public $receiverVolume = 5;
    public $receiverMicroVolume = 5;
    public $headsetVolume = 5;
    public $headsetMicroVolume = 5;
    public $speakerVolume = 5;
    public $speakerMicroVolume = 5;

    public static function myAttributes()
    {
        return array(
            "userId" => "userId",
            "ringVolume" => "ringVolume",
            "receiverVolume" => "receiverVolume",
            "receiverMicroVolume" => "receiverMicroVolume",
            "headsetVolume" => "headsetVolume",
            "headsetMicroVolume" => "headsetMicroVolume",
            "speakerVolume" => "speakerVolume",
            "speakerMicroVolume" => "speakerMicroVolume",
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return User the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'usersettings';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('ringVolume, receiverVolume, receiverMicroVolume, headsetVolume, headsetMicroVolume, speakerVolume, speakerMicroVolume',
                'numerical',
                'integerOnly' => true
            ),
            array('ringVolume, receiverVolume, receiverMicroVolume, headsetVolume, headsetMicroVolume, speakerVolume, speakerMicroVolume, userId',
                'safe'
            ),
        );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array();
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'userId' => __('USERSETTINGS_MODEL_USER_ID'),
            'ringVolume' => __('USERSETTINGS_MODEL_RING_VOLUME'),
            'receiverVolume' => __('USERSETTINGS_MODEL_RECEIVER_VOLUME'),
            'receiverMicroVolume' => __('USERSETTINGS_MODEL_RECEIVER_MICRO_VOLUME'),
            'headsetVolume' => __('USERSETTINGS_MODEL_HEADSET_VOLUME'),
            'headsetMicroVolume' => __('USERSETTINGS_MODEL_HEADSET_MICRO_VOLUME'),
            'speakerVolume' => __('USERSETTINGS_MODEL_SPEAKER_VOLUME'),
            'speakerMicroVolume' => __('USERSETTINGS_MODEL_SPEAKER_MICRO_VOLUME'),
        );
    }
    
    public function setAdminAlertSubscriptions($subscriptions = array())
    {
        foreach ($subscriptions as $subscription => $level)
        {
            if ( $this->validateSubscription($subscription)) {
                $this->setValueForKey($subscription, $level);
            }
        }
        $this->save();
    }
    
    private function validateSubscription($subscription) 
    {
        
        if (in_array($subscription, $this->getAllowedAlertSubscriptions())) {
            return true;
        }
        return false;
    }
    
    public function getAdminAlertSubscriptions()
    {
        $subscriptions = array();
        foreach ($this->getAllowedAlertSubscriptions() as $subscription) {
            $subscriptions[$subscription] = $this->getValueForKey($subscription);
        }
        return $subscriptions;
    }
    
    private function getAllowedAlertSubscriptions()
    {
        return array(
            'alert_Callrouter',
            'alert_Msgd',
            'alert_System',
            'alert_Voipd',
            'alert_WebGUI',
        );
    }
}
