<?php

class DefaultController extends Controller
{
    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout='//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }


    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(


            array('allow',  // allow all users to perform 'index' and 'view' actions
                'actions'=>array('index','view'),
                //'users'=>array('@'),
                'expression'=>AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions'=>array('create','update'),
                //'users'=>array('@'),
                'expression'=>AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions'=>array('admin','delete','JsonBeforeDelete'),
                //'users'=>array('@'),
                'expression'=>AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny',  // deny all users
                'users'=>array('*'),
            ),
        );
    }
    
    /**
     * Displays a particular model.
     * @param integer $id the ID of the model to be displayed
     */
    public function actionView($id)
    {
        $this->render('view',array(
                'model'=>$this->loadModel($id),
            ));
    }


    /**
     * Creates a new model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     */
    public function actionCreate()
    {
        $model=new Addressbook;

        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);

        if(isset($_POST['Addressbook']))
        {
            $model->attributes=$_POST['Addressbook'];
            
            $privateAddressbookCount = Addressbook::model()->countAllByAttributes(array('type'=> AddressbookType::personal));
            if (PhonalisaLicenseManager::privateAddressbooksMax() <= $privateAddressbookCount && PhonalisaLicenseManager::privateAddressbooksMax() !== -1) {
                $model->type = AddressbookType::group;
            }
            
            $publicAddressbooksCount = PhonalisaLicenseManager::publicAddressbooksMax();
            
            
            if(!$model->save()){
                throw new CHttpException(500,'Can not save new data.');
            } else {
                if (isset($_POST['Permissions'])) {
                    $this->savePermissions($model->id);
                    $permissionModel = new Permissions();
                    $permissionModel->createPermissionsFile();
                }

            }
            //$this->redirect(array('view','id'=>$model->id));

        }
        $this->renderPartialWithContextHelp('_form', array('model'=>$model), false, true);
    }


    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id)
    {
        $model=$this->loadModel($id);

        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);

        if(isset($_POST['Addressbook']))
        {
            $model->attributes=$_POST['Addressbook'];
            if(!$model->save()){
                throw new CHttpException(500,'Can not save new data.');
            } else {
                if (isset($_POST['Permissions'])) {
                    $this->savePermissions($model->id);
                    $permissionModel = new Permissions();
                    $permissionModel->createPermissionsFile();
                }
            }

            return true;
            //$this->redirect(array('view','id'=>$model->id));
        }
        $this->renderPartialWithContextHelp('_form', array('model'=>$model), false, true);
    }


    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionJsonBeforeDelete($bookId) {
    
	$addresses = Addresses::model()->findByAttributes(array('contact_tid'=>$bookId));
	if ($addresses) {
	    Yii::app()->user->setFlash('error', '<strong>'.__("ADDRESSBOOK_GENERAL_ADDRESSES_EXISTS_ERROR_MESSAGE").'</strong>');
	    $json = json_encode(false);
	} else {
	    $book = $this->loadModel($bookId);
	    $json = json_encode($book->delete());
	}
        echo $json;
    }


    public function actionDelete($id)
    {

        if(Yii::app()->request->isPostRequest)
        {
            // we only allow deletion via POST request
            $this->loadModel($id)->delete();

            // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if(!isset($_GET['ajax']))
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
        }
        else
            throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
    }


    /**
     * Lists all models.
     */
    public function actionIndex() {
        $this->actionAdmin();
    }


    /**
     * Manages all models.
     */
    public function actionAdmin() {

        $model=new Addressbook('search');

        $model->unsetAttributes();  // clear any default values
        if(isset($_GET['Addressbook'])) {
            $model->attributes=$_GET['Addressbook'];
        }
        


        $this->render('admin',array(
                'model'=>$model,
            ));
    }


    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model=Addressbook::model()->findByPk($id);
        if($model===null)
            throw new CHttpException(404,'The requested page does not exist.');
        return $model;
    }


    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model) {
        if(isset($_POST['ajax']) && $_POST['ajax']==='addressbook-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }


    public function getIconDropdown() {
        $imgPath = Yii::app()->basePath.'/../img/icons/mainMenu/desktop/sidebar/';
        $icons = array();

        foreach (glob($imgPath . "*.png") as $filePath) {
            $fileName = basename($filePath);
            $icons[$fileName] = $fileName;
        }
        asort($icons);
        return $icons;
    }


    public  function savePermissions($modelId) {
        foreach ($_POST['Permissions'] as $groupId => $permissionTypes) {
            foreach ($permissionTypes as $permissionName => $value) {
                $oldPermission = Permissions::model()->findByAttributes(
                    array(
                        'groupId'=>$groupId,
                        'itemId'=>$modelId,
                        'type'=>$permissionName
                    )
                );
                if ($value == '0') {
                    if (count($oldPermission)) {
                        $oldPermission->delete();
                    }
                } else {
                    if (!$oldPermission) {
                        $newPermission = new Permissions();
                        $newPermission->type = $permissionName;
                        $newPermission->groupId =$groupId;
                        $newPermission->itemId =  $modelId;
                        $newPermission->save();
                    }
                }
            }
        }

    }
    
//     public function getAlternativeAddressesLink() {
// 	return 'test';
//         $AM = new AccessManager();
//         
//         $permissions = $AM->getPermissions();
// 
//         if (isset($permissions['read']) && count($permissions['read'])) {
// 
//             foreach ($permissions['read'] as $id => $attributes) {
// 
//                 $firstPermissions = $attributes['itemId'];
//                 break;
//             }
//             if (@$_REQUEST['id'] == $firstPermissions) {
//                     array_shift($permissions['read']);
//             }
//         }
//         foreach ($permissions['read'] as $id => $permission) {
//             if (count($permission)) {
//                 return $permission['itemId'];
//             } else {
//                 return ""; // FIXME ! 
//             }
//         } 
//        
//     }
}
?>