<?php

class DefaultController extends Controller
{

	/**
	 * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
	 * using two-column layout. See 'protected/views/layouts/column2.php'.
	 */
	public $layout='//layouts/column2';

	/**
	 * @return array action filters
	 */
	public function filters()
	{
		return array(
				'accessControl', // perform access control for CRUD operations
		);
	}

	/**
	 * Specifies the access control rules.
	 * This method is used by the 'accessControl' filter.
	 * @return array access control rules
	 */
	public function accessRules()
	{
		return array(
				array('allow',  // allow all users to perform 'index' and 'view' actions
						'actions'=>array('index','view', 'generateData', 'getJsonEvents'),
						//'users'=>array('@'),
						'expression'=>AccessManager::checkSessionPermission($this->module->name),
				),
				array('allow', // allow authenticated user to perform 'create' and 'update' actions
						'actions'=>array('create','update'),
						//'users'=>array('@'),
						'expression'=>AccessManager::checkSessionPermission($this->module->name),
				),
				array('allow', // allow admin user to perform 'admin' and 'delete' actions
						'actions'=>array('admin','delete'),
						//'users'=>array('@'),
						'expression'=>AccessManager::checkSessionPermission($this->module->name),
				),
				array('deny',  // deny all users
						'users'=>array('*'),
				),
		);
	}

	/**
	 * Displays a particular model.
	 * @param integer $id the ID of the model to be displayed
	 */
	public function actionView($id)
	{
		$this->render('view',array(
				'model'=>$this->loadModel($id),
		));
	}

	/**
	 * Creates a new model.
	 * If creation is successful, the browser will be redirected to the 'view' page.
	 */
	public function actionCreate()
	{
		$model=new Calendar();
		
		// Uncomment the following line if AJAX validation is needed
		 $this->performAjaxValidation($model);
		 
		if(isset($_POST['Calendar']))
		{

			$model->attributes=$_POST['Calendar'];
			if($model->save()) {
				if (isset($_POST['Permissions'])) {
					$this->savePermissions($model->id);
                    $permissionModel = new Permissions();
                    $permissionModel->createPermissionsFile();
				}
			}
				
		}
		$this->renderPartialWithContextHelp('_form', array('model'=>$model), false, true);
	}

	/**
	 * Updates a particular model.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 * @param integer $id the ID of the model to be updated
	 */
	public function actionUpdate($id)
	{

		$model=$this->loadModel($id);

		// Uncomment the following line if AJAX validation is needed
		 $this->performAjaxValidation($model);

		if(isset($_POST['Calendar']))
		{
            
            if (isset($_POST['Calendar']['type'])) {
                unset($_POST['Calendar']['type']);
            } 
            
			$model->attributes=$_POST['Calendar'];
			if($model->save()) {
				if (isset($_POST['Permissions'])) {
					$this->savePermissions($model->id);
                    $permissionModel = new Permissions();
                    $permissionModel->createPermissionsFile();
				}
			}
			return true;
		}

		$this->renderPartialWithContextHelp('_form', array('model'=>$model), false, true);
	}
	
	public  function savePermissions($modelId) {
		foreach ($_POST['Permissions'] as $groupId => $permissionTypes) {
			foreach ($permissionTypes as $permissionName => $value) {
				$oldPermission = Permissions::model()->findByAttributes(
						array(
								'groupId'=>$groupId,
								'itemId'=>$modelId,
								'type'=>$permissionName
						)
				);
				if ($value == '0') {
					if (count($oldPermission)) {
						$oldPermission->delete();
					}
				} else {
					if (!$oldPermission) {
						$newPermission = new Permissions();
						$newPermission->type = $permissionName;
						$newPermission->groupId =$groupId;
						$newPermission->itemId =  $modelId;
						$newPermission->save();
					}
				}
			}
		}
		 
	}

	/**
	 * Deletes a particular model.
	 * If deletion is successful, the browser will be redirected to the 'admin' page.
	 * @param integer $id the ID of the model to be deleted
	 */
	public function actionDelete($id)
	{
		if(Yii::app()->request->isPostRequest)
		{
			// we only allow deletion via POST request
			$this->loadModel($id)->delete();

			// if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
				
			if(!isset($_GET['ajax'])){
				$this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
			}

		}
		else
			throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
	}

	/**
	 * Lists all models.
	 */
	public function actionIndex()
	{
		
		$this->actionAdmin();
		
	}


	/**
	 * Manages all models.
	 */
	public function actionAdmin()
	{

		$model=new Calendar('search');
		$model->unsetAttributes();  // clear any default values
		if(isset($_GET['Calendar']))
			$model->attributes=$_GET['Calendar'];

		$this->render('admin',array(
				'model'=>$model,
		));
	}

	/**
	 * Returns the data model based on the primary key given in the GET variable.
	 * If the data model is not found, an HTTP exception will be raised.
	 * @param integer the ID of the model to be loaded
	 */
	public function loadModel($id)
	{
		$model= Calendar::model()->findByPk($id);
		if($model===null)
			throw new CHttpException(404,'The requested page does not exist.');
		return $model;
	}

	/**
	 * Performs the AJAX validation.
	 * @param CModel the model to be validated
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='calendar-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}
	
	
	protected function renderGridCellColor($data, $row){     
        return $this->renderPartial('gridCellColor', array('data'=>$data, 'row' => $row));
    }

}
