<?php
class DefaultController extends Controller
{
    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */

    // Disable custom layout
    public $layout='//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }


    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow',  // allow all users to perform 'index' and 'view' actions
                'actions'=>array('index','view', 'monitor'),
                //'users'=>array('@')
                'expression'=>AccessManager::checkSessionPermission($this->module->id),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions'=>array('create','update', 'copy'),
                'users'=>array('@'),
                'expression'=>AccessManager::checkSessionPermission($this->module->id),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions'=>array('admin','delete'),
                //'users'=>array('@'),
                'expression'=>AccessManager::checkSessionPermission($this->module->id),
            ),
            array('deny',  // deny all users
                'users'=>array('*'),
            ),
        );
    }




    public function actionIndex(){
        $model = ForwardPreset::model();    
        $criteria = new CDbCriteria;
        $filter = $model->getFilter($criteria, 'name');
    	$filter['criteria']->condition .= 'type='.ForwardPresetType::PublicPreset;

        $dataProvider = new CPhonalisaDataProvider('ForwardPreset', $filter);

        $this->render('presets',array(
            'dataProvider' => $dataProvider,
            'model' => $model,
        ));
    }


    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id) {
        $model = ForwardPreset::model()->findByPk($id);


        if(isset($_POST['ForwardPreset'])) {
            $model->attributes = $_POST['ForwardPreset'];
            if($model->save()) {
                Yii::app()->user->setFlash('success', __('CHANGES_SAVED_SUCCSESSFULLY'));
                $this->redirect(array('index'));
            } else {
                Yii::app()->user->setFlash('error', __('CHANGES_NOT_SAVED_CAUSE_OF_AN_ERROR'));
            }
        }

        $this->render('_form',array(
            'model'=>$model,
            'write' => true,
        ));
    }

    
    
    public function actionCreate() {
        $model = ForwardPreset::model();
        $model->type = ForwardPresetType::PublicPreset;

        if(isset($_POST['ForwardPreset'])) {
            $model->attributes = $_POST['ForwardPreset'];
            if($model->save()) {
                if($model->forwardType == ForwardType::Disabled){
                    Yii::app()->user->setFlash('success', __('CHANGES_SAVED_SUCCSESSFULLY'));
                    $this->redirect(array('index'));
                }
                $this->redirect(array('update', 'id' => $model->id));
            }
        }

        $this->render('_form',array(
                'model'=>$model,
                'write' => true,
            ));
    }
    
    
    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model=ForwardPreset::model()->findByPk($id);
        if($model===null) {
            throw new CHttpException(404,'The requested page does not exist.');
        }
        return $model;
    }



    
    public function actionDelete($id)
    {
        if(Yii::app()->request->isPostRequest) {
            // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if(isset($_GET['ajax'])){
                $this->loadModel($id)->delete();
            }

        } else {
            throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
        }
    }

    public function getForwardTypeDropdownListData()
    {
        $listData = array(
            ForwardType::Immediately => ForwardType::translateType(ForwardType::Immediately),
            ForwardType::Custom => ForwardType::translateType(ForwardType::Custom)
        );

        $defaultCallforward = ForwardPreset::model()->findByAttributes(
                array(
                    'forwardType' => ForwardType::Disabled,
                    'type' => ForwardPresetType::PublicPreset
                )
        );

        if (!$defaultCallforward) {
            $listData[ForwardType::Disabled] = ForwardType::translateType(ForwardType::Disabled);
        }
        return $listData;
    }
}
