<?php

class ConfigurationOverview
{

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return adminCdr the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
        );
    }

    public function getModelDataFromGateways()
    {
        return Gateway::model()->findAllByAttributes(array('active' => '1'));
    }

    public function getRevisedModelValue($modelName, $modelKey, $value)
    {
        $valueWithChangedBool = ConfigurationOverview::changeBoolValuesToHumanReadable($modelName, $modelKey, $value);
        $valueWithChangedMoh = ConfigurationOverview::changeMusicOnHoldToHumanReadableText($modelKey, $valueWithChangedBool);
        $valueWithChangedIdText = ConfigurationOverview::changeIdToHumanReadableText($modelKey, $valueWithChangedMoh);
        $valueWithChangedTarget = ConfigurationOverview::changeTargetNamesToHumanReadable($modelKey, $valueWithChangedIdText);

        $result = $valueWithChangedTarget;

        return $result;
    }

    protected function changeBoolValuesToHumanReadable($modelName, $modelKey, $modelValue)
    {
        $modelValueArray = ConfigurationOverview::getTooltipModelKeysWithBoolValues($modelName);
        $result = $modelValue;

        if (in_array($modelKey, $modelValueArray)) {
            if ($modelValue == '0' || $modelValue == 'false') {
                $result = __('GLOBAL_NO');
            } else if ($modelValue == '1' || $modelValue == 'true') {
                $result = __('GLOBAL_YES');
            }
        }
        return $result;
    }

    protected function getTooltipModelKeysWithBoolValues($modelName)
    {
        $result = array();
        if ($modelName == 'Gateway') {
            $result = ConfigurationOverview::getTooltipDataForGateways();
        }
        if ($modelName == 'User') {
            $result = ConfigurationOverview::getTooltipDataForUsers();
        }
        if ($modelName == 'Groups') {
            $result = ConfigurationOverview::getTooltipDataForGroups();
        }
        if ($modelName == 'CallGroup') {
            $result = ConfigurationOverview::getTooltipDataForCallGroups();
        }
        if ($modelName == 'Queue') {
            $result = ConfigurationOverview::getTooltipDataForQueues();
        }
        if ($modelName == 'Conference') {
            $result = ConfigurationOverview::getTooltipDataForConference();
        }
        return $result;
    }

    protected function getTooltipDataForGateways()
    {
        return array(
            'active',
            'register',
            'dialOutToggle',
            'caller_id_in_from',
            'tcp_transport',
            'calldeflect_support',
            'answer_on_outbound_dial',
        );
    }

    protected function getTooltipDataForUsers()
    {
        return array(
            'zrtp',
            'isSystemAdmin',
            'userMonitor'
        );
    }

    protected function getTooltipDataForGroups()
    {
        return array('night_circut', 'accountingEnabled');
    }

    protected function getTooltipDataForCallGroups()
    {
        return array('night_circut');
    }

    protected function getTooltipDataForQueues()
    {
        return array(
            'tier_rules_apply',
            'tier_rule_wait_multiply_level',
            'tier_rule_no_agent_no_wait',
            'abandoned_resume_allowed',
        );
    }

    protected function getTooltipDataForConference()
    {
        return array(
            'listen_to_music',
            'useConfereceMemberName'
        );
    }

    protected function changeMusicOnHoldToHumanReadableText($modelKey, $modelValue)
    {
        $result = $modelValue;
        $modelKeyArray = array('mohId', 'playlistId', 'announce_file');

        if (in_array($modelKey, $modelKeyArray)) {
            $audioModel = TreeMapHelper::getModelFromTarget($modelValue, true);

            if ($modelValue == 'default') {
                $result = __("DEFAULT_MOH_PLAYLIST_NAME");
            }
            if ($modelValue == '0') {
                $result = __("ROUTING_CALL_GROUP_RING_INSTEAD_OF_MUSIC");
            }
            $result = ConfigurationOverview::getLabelForModelObject($audioModel);
        }
        return $result;
    }

    protected function changeIdToHumanReadableText($modelKey, $modelValue)
    {
        $result = $modelValue;
        $modelKeyArray = array('group', 'callForwardSet', 'parent', 'defaultCallForwardSet');

        if (in_array($modelKey, $modelKeyArray)) {
            $model = TreeMapHelper::getModelFromTarget($modelValue, true);
            $result = ConfigurationOverview::getLabelForModelObject($model);
        }
        return $result;
    }

    protected function changeTargetNamesToHumanReadable($modelKey, $modelValue)
    {
        $result = $modelValue;
        $modelKeyArray = array('target_on_timeout', 'target_on_busy', 'target_on_nightcircut',);
        if (in_array($modelKey, $modelKeyArray)) {

            $model = TreeMapHelper::getModelFromTarget($modelValue, true);
            $result = ConfigurationOverview::getLabelForModelObject($model);
        }
        return $result;
    }

    protected function getLabelForModelObject($model)
    {
        $result = '';
        if ($model) {
            if (isset($model->name)) {
                $result = $model->name;
            } else {
                $result = $model->title;
            }
        }
        return $result;
    }
}
