<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return list of available group types
     */

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            //TEST
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('GetJSONParentPermissions'),
                'users' => array('@'),
            ),
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'view'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions' => array('create', 'update', 'detail'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions' => array('admin', 'delete', 'ringMap'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    /**
     * Displays a particular model.
     * @param integer $id the ID of the model to be displayed
     */
    public function actionView($id)
    {
        $this->render('view', array(
            'model' => $this->loadModel($id),
        ));
    }

    /**
     * Creates a new model.
     * If creation is successful, the browser will be redirected to the 'view' page.
     */
    public function actionCreate()
    {
        $model = new Groups('create');

        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);

        if (isset($_POST['Groups'])) {
            $model->attributes = $this->getDataForModelAttributes();
            TargetHelper::setTargetAttributes('target_type_on_timeout', 'target_on_timeout', $model);
            TargetHelper::setTargetAttributes('target_type_on_busy', 'target_on_busy', $model);
            TargetHelper::setTargetAttributes('target_type_on_nightcircut', 'target_on_nightcircut', $model);

            if ($model->save()) {
                if (isset($_POST['Permissions']['new'])) {
                    foreach ($_POST['Permissions']['new'] as $permissionType => $value) {
                        if ($value == '0') {
                            continue;
                        }
                        $permissionsModel = new Permissions;
                        $permissionsModel->type = $permissionType;
                        $permissionsModel->groupId = $model->id;
                        $permissionsModel->itemId = $model->id;
                        $permissionsModel->save();
                    }
                }
                $this->actionUpdate($model->id);
            } else {
                throw new CHttpException(500, 'Can not save new data.');
            }
        }

        $this->renderPartial('_form', array('model' => $model->with('permissions')), false, true);
    }

    protected function getDataForModelAttributes()
    {
        $data = $_POST['Groups'];
        if (PhonalisaLicenseManager::routingBlacklistEnabled()) {
            $inboundListID = $data['inboundBlacklistId'];
            $outboundListID = $data['outboundBlacklistId'];
            $data['inboundBlacklistId'] = '';
            $data['outboundBlacklistId'] = '';
            $data['inboundWhitelistId'] = '';
            $data['outboundWhitelistId'] = '';
            $data = $this->getFormatedBlacklistWhitelistID($data, $inboundListID, 'inbound');
            $data = $this->getFormatedBlacklistWhitelistID($data, $outboundListID, 'outbound');
        }
        return $data;
    }

    protected function getFormatedBlacklistWhitelistID($data, $listID, $listType)
    {
        $listKey = 'whitelist';
        if (strpos($listID, 'blacklist_') !== false) {
            $listKey = 'blacklist';
        }
        $data[$listType . ucfirst($listKey) . 'Id'] = str_replace($listKey . '_', '', $listID);
        return $data;
    }

    public function saveMembers($model)
    {

        // remove Member
        $defaultGroup = Groups::model()->findByAttributes(array('type' => 'allUsers'));
        $currentMembers = $model->users;
        if (isset($_POST['User'])) {
            foreach ($currentMembers as $member) {
                foreach ($_POST['User'] as $key => $data) {
                    $remove = true;
                    if ($member->id == $data['userId']) {
                        $remove = false;
                    }
                }
                if ($remove == true) {
                    $member->group = $defaultGroup->id;
                    $member->save();
                }
            }
        } else {
            foreach ($currentMembers as $member) {
                $member->group = $defaultGroup->id;
                $member->save();
            }
        }


        //add new Members
        if (isset($_POST['User'])) {

            foreach ($_POST['User'] as $row => $data) {

                $member = User::model()->findByPk($data['userId']);
                if ($member) {
                    if ($member->group != $model->id) {
                        $member->group = $model->id;
                        $member->save();
                    }
                }
            }
        }
    }

    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'view' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id)
    {

        $model = $this->loadModel($id);
        $permissions = $model->permissions;
        $model = $this->changeBlacklistWhitelistSettings($model);
        if (!isset($model->faxSendAllowed)) {
            $model->faxSendAllowed = User::FAX_SEND_INHERIT_FROM_GROUP;
            if ($model->type === Groups::GROUP_TYPE_ALL_USERS) {
                $model->faxSendAllowed = User::FAX_SEND_ALLOWED;
            }
        }

        // Uncomment the following line if AJAX validation is needed
        $this->performAjaxValidation($model);
        if (isset($_POST['Groups'])) {

            $oldId = $model->id;
            $model->attributes = $this->getDataForModelAttributes();
            TargetHelper::setTargetAttributes('target_type_on_timeout', 'target_on_timeout', $model);
            TargetHelper::setTargetAttributes('target_type_on_busy', 'target_on_busy', $model);
            TargetHelper::setTargetAttributes('target_type_on_nightcircut', 'target_on_nightcircut', $model);
            if ($model->save()) {

                $this->saveMembers($model);

                // save Group Permissions
                foreach ($_POST['Permissions'] as $itemId => $permissions) {

                    foreach ($permissions as $permissionType => $value) {
                        $oldPermission = Permissions::model()->findByAttributes(
                                array(
                                    'groupId' => $model->id,
                                    'itemId' => $itemId,
                                    'type' => $permissionType,
                                )
                        );
                        if ($value == '0') {
                            if ($oldPermission) {
                                $oldPermission->delete();
                                //continue;
                            }
                        } else {
                            if (!$oldPermission) {
                                $newPermissions = new Permissions();
                                $newPermissions->type = $permissionType;
                                $newPermissions->groupId = $model->id;
                                $newPermissions->itemId = $itemId;
                                $newPermissions->save();
                            }
                        }
                    }
                }
                $permissionModel = new Permissions();
                $permissionModel->createPermissionsFile();

                $model->updateNestedIndex();
                // Render result after succsessful save

                $this->actionAdmin();
                Yii::app()->end();
            } else {
                throw new CHttpException(500, 'Can not save new data.');
            }
        }
        $this->renderPartial('_form', array('model' => $model), false, true);
    }

    protected function changeBlacklistWhitelistSettings($model)
    {
        if ($model->inboundBlacklistId !== '') {
            $model->inboundBlacklistId = 'blacklist_' . $model->inboundBlacklistId;
        }
        if ($model->outboundBlacklistId !== '') {
            $model->outboundBlacklistId = 'blacklist_' . $model->outboundBlacklistId;
        }
        if ($model->inboundWhitelistId !== '') {
            $model->inboundBlacklistId = 'whitelist_' . $model->inboundWhitelistId;
            $model->inboundWhitelistId = '';
        }
        if ($model->outboundWhitelistId !== '') {
            $model->outboundBlacklistId = 'whitelist_' . $model->outboundWhitelistId;
            $model->outboundWhitelistId = '';
        }
        return $model;
    }

    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            // we only allow deletion via POST request
            $this->loadModel($id)->delete();

            // Remove all permissions for this id or on this id
            #foreach(Permissions::model()->findAll("`itemId`=\"".$id."\" OR `groupId`=\"".$id."\"") as $permission){
            foreach (Permissions::model()->findAllByAttributes(array('groupId' => $id)) as $permission) {
                $permission->delete();
            }
            foreach (Permissions::model()->findAllByAttributes(array('itemId' => $id)) as $permission) {
                $permission->delete();
            }

            // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    /**
     * Lists all models.
     */
    public function actionIndex()
    {
        $this->actionAdmin();
    }

    public function actionDetail($id)
    {

        $criteria = new CDbCriteria;
        $criteria->order = 'name';
        $criteria->condition = 'group=' . $id;

        $dataProvider = new CPhonalisaDataProvider('User', array('criteria' => $criteria));

        $this->render('detail', array(
            'group' => Groups::model()->findByPk($id),
            'model' => User::model(),
            'dataProvider' => $dataProvider,
        ));
    }

    /**
     * Manages all models.
     */
    public function actionAdmin()
    {
        $this->updateNestedIndexIfIndexNotFound();
        if (isset($_GET['type']) && strlen($_GET['type']) !== 0) {
            $model = $this->loadGroupModel($_GET['type']);
        } else {
            $model = Groups::model();
        }

        $this->render('admin', array(
            'model' => $model,
        ));
    }

    protected function updateNestedIndexIfIndexNotFound()
    {
        $groups = Groups::model()->findAll();
        $nestedGroupsIndex = array();

        foreach ($groups as $group) {
            $nestedGroupsIndex[$group->nestedIndex] = $group->nestedName;
        }
        ksort($nestedGroupsIndex);
        $firstIndex = key($nestedGroupsIndex);
        end($nestedGroupsIndex);
        $lastIndex = key($nestedGroupsIndex);

        $indexNeedUpdate = false;
        for ($i = $firstIndex; $i <= $lastIndex; $i++) {
            if (!isset($nestedGroupsIndex[$i])) {
                $indexNeedUpdate = true;
                break;
            }
        }
        if ($indexNeedUpdate) {
            Groups::model()->updateNestedIndex();
        }
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = Groups::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    public function loadGroupModel($group)
    {
        $model = new Groups('search');

        $criteria = $model->getDbCriteria(true);
        $criteria->addCondition(array('type' => $group));
        $model->setDbCriteria($criteria);

        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'groups-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }

    public function getPermissions($model)
    {
        $dataProvider = new CArrayDataProvider($model->permissions, array(
            'id' => 'id',
            'sort' => array(
                'attributes' => array('id'),
            ),
        ));
        return $dataProvider;
    }

    public function getPermissionTypes($groupType)
    {
        $groups = $this->availableGroupTypes();
        return $groups[$groupType]['permissionTypes'];
    }

    public function loadPermissionByAttributes($permissionType, $itemId, $groupId)
    {
        return Permissions::model()->findByAttributes(
                        array(
                            'type' => $permissionType,
                            'itemId' => $itemId,
                            'groupId' => $groupId
                        )
        );
    }

    public function loadPermissionsByItemId($itemId)
    {
        return Permissions::model()->findByAttributes(array('itemId' => $itemId));
    }

    public function loadPermissionsByGroupId($groupId)
    {
        return Permissions::model()->findByAttributes(array('groupId' => $groupId));
    }

    public function getTabularFormTabs($form, $model)
    {
        $tabs = array();
        $count = 0;

        $translateKeys = array();
        $translateKeys["generalData"] = __('USERGROUPS_MODEL_TAB_MENU_GENERAL_DATA');
        $translateKeys["membersTab"] = __('USERGROUPS_MODEL_TAB_MENU_GROUP_MEMBERS');
        $translateKeys["groupPermissionsGrid"] = __('USERGROUPS_MODEL_TAB_MENU_GROUP_PERMISSIONS');
        $translateKeys["queuePermissions"] = __('USERGROUPS_MODEL_TAB_MENU_QUEUE_PERMISSIONS');
        $translateKeys["addressbookPermissionsGrid"] = __('USERGROUPS_MODEL_TAB_MENU_ADDRESSBOOK_PERMISSION');
        $translateKeys["calendarPermissionsGrid"] = __('USERGROUPS_MODEL_TAB_MENU_CALENDAR_PERMISSION');
        $translateKeys["forwardSetPermissionsGrid"] = __('USERGROUPS_MODEL_TAB_MENU_FORWARD_SETS_PERMISSION');
        $translateKeys["advancedTab"] = __('USERGROUPS_MODEL_TAB_MENU_ADVANCED');

        $this->filterFormularTabs($translateKeys);

        foreach ($translateKeys as $view => $tabLabel) {
            $tabs[] = array(
                'active' => $count++ === 0,
                'label' => $tabLabel,
                'content' => $this->renderTabContent($view, $model, $form)
            );
        }
        return $tabs;
    }

    private function filterFormularTabs(&$translateKeys)
    {
        if (!$this->queueTabEnabledByLicence()) {
            unset($translateKeys["queuePermissions"]);
        }

        if (!PhonalisaLicenseManager::adminCalendarEnabled()) {
            unset($translateKeys["calendarPermissionsGrid"]);
        }

        if (!PhonalisaLicenseManager::adminCallForwardingSetsEnabled()) {
            unset($translateKeys["forwardSetPermissionsGrid"]);
        }
    }

    private function queueTabEnabledByLicence()
    {
        $enabled = true;
        if (PhonalisaLicenseManager::routingQueuesMax() === 0) {
            $enabled = false;
        }
        if (!PhonalisaLicenseManager::analyticsQueueReportEnabled() && !PhonalisaLicenseManager::adminQueueMonitorEnabled()) {
            $enabled = false;
        }
        return $enabled;
    }

    private function renderTabContent($view, $model, $form)
    {
        if ($this->hasRelevantModels($view)) {
            return $this->renderPartialWithContextHelp($view, array('form' => $form, 'model' => $model), true);
        }
        return $this->renderPartial('notFound', array('modelName' => $this->getTabModelName($view)), true);
    }

    private function getTabModelName($view)
    {
        $tabModelName = $view;
        if ($view === 'queuePermissions') {
            $tabModelName = "Queue";
        } else if ($view === 'calendarPermissionsGrid') {
            $tabModelName = 'Calendar';
        } else if ($view === 'forwardSetPermissionsGrid') {
            $tabModelName = 'ForwardPreset';
        }
        return $tabModelName;
    }

    private function hasRelevantModels($view)
    {
        if ($view === "membersTab") {
            return User::model()->count() > 1;
        } else if ($view === "groupPermissionsGrid") {
            return Groups::model()->count() > 0;
        } else if ($view === "queuePermissions") {
            return Queue::model()->count() > 0;
        } else if ($view === "addressbookPermissionsGrid") {
            return Addressbook::model()->count() > 0;
        } else if ($view === "calendarPermissionsGrid") {
            return Calendar::model()->count() > 0;
        } else if ($view === "forwardSetPermissionsGrid") {
            return ForwardPreset::model()->count() > 0;
        }
        return true;
    }

    public function renderUserColumn($data, $row, $column)
    {
        $this->renderPartial('memberUserColumn', array('data' => $data, 'row' => $row, 'column' => $column));
    }

    public function actionRingMap($id)
    {


        if (isset($_POST['TimeslotsUser'])) {
            //dump($_POST['Timeslots']);

            foreach ($_POST['TimeslotsUser'] as $userId => $values) {
                $user = User::model()->findByPk($userId);

                if ($user == null) {
                    Log::e("User $userId not found", "phonalisa-webinterface-groups");
                    continue;
                }


                $keyPrefix = 'gc_' . $id . '_';
                $user->setValueForKey($keyPrefix . 'start', $values['start']);
                $user->setValueForKey($keyPrefix . 'end', $values['end']);
                $user->save();
            }
        }

        if (isset($_POST['TimeslotsGroups'])) {

            foreach ($_POST['TimeslotsGroups'] as $groupId => $values) {
                $_group = Groups::model()->findByPk($groupId);

                if ($_group == null) {
                    Log::e("User $groupId not found", "phonalisa-webinterface-groups");
                    continue;
                }

                $keyPrefix = 'gc_' . $id . '_';
                $_group->setValueForKey($keyPrefix . 'start', $values['start']);
                $_group->setValueForKey($keyPrefix . 'end', $values['end']);
                $_group->save();
            }
        }

        $group = Groups::model()->findByPk($id);
        if (!isset($group->maxRingTime) || $group->maxRingTime == '') {
            $group->maxRingTime = Groups::DEFAULT_MAX_RING_TIME;
        }

        $criteria = new CDbCriteria;
        $criteria->order = 'name';
        $criteria->condition = 'group=' . $id;

        $userDataProvider = new CPhonalisaDataProvider(
                'User', array(
            'criteria' => $criteria,
            'pagination' => array('pageSize' => 1000)
                )
        );

        $groupsDataProvider = new CArrayDataProvider($group->getChildren(true), array(
            'sort' => array(
                'defaultOrder' => 'nestedIndex ASC',
                'attributes' => array(
                //'nestedName',
                ),
            ),
            'pagination' => false
        ));

        $this->render('ringMap', array(
            'group' => $group,
            'userModel' => User::model(),
            'userDataProvider' => $userDataProvider,
            'groupsModel' => Groups::model(),
            'groupsDataProvider' => $groupsDataProvider,
        ));
    }

    public function renderRingTimeSlider($data, $row, $column)
    {
        $this->renderPartial('ringTime', array('data' => $data, 'row' => $row, 'column' => $column));
    }

    public function actionGetJSONParentPermissions()
    {
        $permissionModel = new Permissions();
        $permissionsArray = array();

        $currentGroup = null;
        if (isset($_POST['modelId']) && $_POST['modelId'] !== "") {
            $currentGroup = Groups::model()->findByPk($_POST['modelId']);
        }

        if (isset($_POST['parentId'])) {
            $permissionsArray = $this->processPermissionsIfParentIdIsSet($currentGroup, $permissionModel);
        } else {
            $permissionsArray = $this->processPermissionIfParentIdIsNotSet($currentGroup, $permissionModel);
        }
        echo json_encode($permissionsArray);
    }

    protected function processPermissionsIfParentIdIsSet($currentGroup, $permissionModel)
    {
        $permissionsArray = array();
        $parentGroup = Groups::model()->findByPk($_POST['parentId']);
        $groups = $parentGroup->getParentGroups();
        $groups[] = $parentGroup;
        $mergedPermissions = array();
        $accountingEnabledFromParent = false;
        $accountingEnabledFromSelf = false;

        foreach ($groups as $group) {
            if ($group->accountingEnabled === '1') {
                $accountingEnabledFromParent = true;
            }
            if (isset($currentGroup) && $currentGroup->accountingEnabled === '1') {
                $accountingEnabledFromSelf = true;
            }
            $parentPermissions = $permissionModel->getPermissionsArray(array('groupId' => $group->id));
            $mergedPermissions = array_merge($mergedPermissions, $parentPermissions);
        }

        $permissionsArray['parent'] = $mergedPermissions;
        $permissionsArray['accountingEnabledFromParent'] = $accountingEnabledFromParent;
        $permissionsArray['accountingEnabledFromSelf'] = $accountingEnabledFromSelf;

        return $this->processPermissionsForCurrentGroup($currentGroup, $permissionsArray, $permissionModel);
    }

    protected function processPermissionsForCurrentGroup($currentGroup, $permissionsArray, $permissionModel)
    {
        if ($currentGroup) {
            $permissionsArray['child'] = $permissionModel->getPermissionsArray(array('groupId' => $currentGroup->id));
            foreach ($permissionsArray['parent'] as $attributes) {
                $permissionsArray = $this->processChildPermissions($permissionsArray, $attributes);
            }
        }
        return $permissionsArray;
    }

    protected function processChildPermissions($permissionsArray, $attributes)
    {
        foreach ($permissionsArray['child'] as $childPermissionId => $childAttributes) {
            if ($childAttributes['itemId'] == $attributes['itemId'] && $childAttributes['type'] == $attributes['type']) {
                unset($permissionsArray['child'][$childPermissionId]);
            }
        }
        return $permissionsArray;
    }

    protected function processPermissionIfParentIdIsNotSet($currentGroup, $permissionModel)
    {
        $permissionsArray = array();
        if (isset($currentGroup) && $currentGroup != null) {
            $permissionsArray['child'] = $permissionModel->getPermissionsArray(array('groupId' => $currentGroup->id));
            $permissionsArray['accountingEnabled'] = $currentGroup->accountingEnabled;
        }
        return $permissionsArray;
    }

    public function getDataProvider($model)
    {
        $users = $model->users;
        if (!count($users)) {
            $users = array(new User());
        }
        return new CArrayDataProvider($users, array('pagination' => false));
    }

    public function getDeleteVisibleStatus($model)
    {
        $deleteVisible = "false";
        if ($model->type != "allUsers") {
            $deleteVisible = "true";
        }
        return $deleteVisible;
    }

    public function getDeletePointer($model)
    {
        $deleteVisible = "default";
        if ($model->type != "allUsers") {
            $deleteVisible = "pointer";
        }
        return $deleteVisible;
    }

    public function getDeleteTooltipTitle($model)
    {
        $deleteVisible = "";
        if ($model->type != "allUsers") {
            $deleteVisible = __("GRID_DELETE_TOOLTIP");
        }
        return $deleteVisible;
    }

    public function getListDataForMembersDropdown($model)
    {
        $members = $model->getGroupMembers();

        $listData = array();
        $availableUsers = User::model()->find(array(), 'name');
        foreach ($availableUsers as $key => $user) {
            if (array_key_exists($user->id, $members) || $user->id == $model->id) {
                unset($availableUsers[$key]);
            } else {
                $listData[$user->id] = getFormatedUsername($user);
            }
        }
        return $listData;
    }

    public function getProvisioningProfileData()
    {
        $lData = array('' => __('GROUPS_ADVANCED_TAB_CHOOSE_PROVISIONING_PROFILE'));
        $listData = CHtml::listData(ProvisioningProfile::model()->find(array(), 'name'), 'id', 'name');
        return array_merge($lData, $listData);
    }

    public function getKeyProfileData()
    {
        $lData = array('' => __('GROUPS_ADVANCED_TAB_CHOOSE_KEY_PROFILE'));
        $listData = CHtml::listData(ProvisioningKeyProfile::model()->find(array(), 'name'), 'id', 'name');
        return array_merge($lData, $listData);
    }

    public function getSpeedDialListData()
    {
        $lData = array('' => __('GROUPS_ADVANCED_TAB_CHOOSE_SPEED_DIAL_LIST'));
        $listData = CHtml::listData(SpeedDialList::model()->find(array(), 'name'), 'id', 'name');
        return array_merge($lData, $listData);
    }

    public function getBlacklistData()
    {
        $blacklist = array();
        foreach (CHtml::listData(Blacklist::model()->find(array(), 'name'), 'id', 'name') as $blacklistID => $blacklistName) {
            $blacklist['blacklist_' . $blacklistID] = $blacklistName;
        }

        $whitelist = array();
        foreach (CHtml::listData(Whitelist::model()->find(array(), 'name'), 'id', 'name') as $whitelistID => $whitelistName) {
            $whitelist['whitelist_' . $whitelistID] = $whitelistName;
        }

        return array(
            '' => '---',
            __('GROUPS_ADVANCED_TAB_CHOOSE_BLACKLIST') => $blacklist,
            __('GROUPS_ADVANCED_TAB_CHOOSE_WHITELIST') => $whitelist
        );
    }
}
