<?php

class DefaultController extends Controller
{
	/**
	 * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
	 * using two-column layout. See 'protected/views/layouts/column2.php'.
	 */
	public $layout='//layouts/column2';

	/**
	 * @return array action filters
	 */
	public function filters()
	{
		return array(
			'accessControl', // perform access control for CRUD operations
		);
	}

	/**
	 * Specifies the access control rules.
	 * This method is used by the 'accessControl' filter.
	 * @return array access control rules
	 */
	public function accessRules()
	{
		return array(
			array('allow',  // allow all users to perform 'index' and 'view' actions
				'actions'=>array('index','view',"ajaxSetCreateBtnVisibility"),
				//'users'=>array('@'),
				'expression'=>AccessManager::checkSessionPermission($this->module->name),
			),
			array('allow', // allow authenticated user to perform 'create' and 'update' actions
				'actions'=>array('create','update'),
				//'users'=>array('@'),
				'expression'=>AccessManager::checkSessionPermission($this->module->name),
			),
			array('allow', // allow admin user to perform 'admin' and 'delete' actions
				'actions'=>array('admin','delete'),
				//'users'=>array('@'),
				'expression'=>AccessManager::checkSessionPermission($this->module->name),
			),
			array('deny',  // deny all users
				'users'=>array('*'),
			),
		);
	}

	/**
	 * Displays a particular model.
	 * @param integer $id the ID of the model to be displayed
	 */
	public function actionView($id)
	{
		$this->render('view',array(
			'model'=>$this->loadModel($id),
		));
	}

	/**
	 * Creates a new model.
	 * If creation is successful, the browser will be redirected to the 'view' page.
	 */
	public function actionCreate()
	{
		$model=new SipAccounts("create");
		$this->performAjaxValidation($model);

		if (isset($_POST['SipAccounts'])) {
			$model->attributes=$_POST['SipAccounts'];
			if (!$model->save()){
    			throw new CHttpException(500,'Can not save new data.');
			}	
			$this->redirect(array('admin','id'=>$model->id));
    			
		}
		$this->renderPartialWithContextHelp('_form', array('model'=>$model,'sortedListData'=>$this->getUserListData($model)), false, true);
	}
	
    public function getUserListData($sipAccount= null)
    {        
        $users = User::model()->find(array(),'extension');
        $listData = array();
        if (!empty($users)) {

            foreach ($users as $user) {
                
                if ($sipAccount && $sipAccount->scenario == "update" && $sipAccount->user_id == $user->id) {
                    $listData[(int)$user->extension][$user->id] = getFormatedUsername($user);
                }
                
                if ($this->userHasMaximumSipaccounts($user)) {
                    continue;
                }
                $listData[(int)$user->extension][$user->id] = getFormatedUsername($user);
            }
        }
        
        $sortedListData = $this->sortUserListData($listData);
        
        return $sortedListData;
    }
    
    
    private function userHasMaximumSipaccounts($user)
    {
        if (PhonalisaLicenseManager::userSipAccountsMax() === -1 ) {
            return false;
        } elseif (PhonalisaLicenseManager::userSipAccountsMax() <= $user->getSipAccountCount()) {
            return true;
        }
        return false;
    }
    
    private function sortUserListData($listData)
    {
        $sortedListData = array();
        
        if (count($listData)) {
            ksort($listData);
            foreach ($listData as $data) {
                foreach ($data as $key => $value) {

                    $sortedListData[$key] = $value;
                }
            }
        }
        return $sortedListData;
    }

	/**
	 * Updates a particular model.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 * @param integer $id the ID of the model to be updated
	 */
	public function actionUpdate($id)
	{
		$model=$this->loadModel($id);

		// Uncomment the following line if AJAX validation is needed
		$this->performAjaxValidation($model);

		if(isset($_POST['SipAccounts']))
		{
			$model->attributes=$_POST['SipAccounts'];
			if(!$model->save()){
                            throw new CHttpException(500,'Can not save new data.');
			}
			return true;
			//$this->redirect(array('view','id'=>$model->id));
		}

		$this->renderPartialWithContextHelp('_form', array('model'=>$model,'sortedListData'=>$this->getUserListData($model)), false, true);
	}

	/**
	 * Deletes a particular model.
	 * If deletion is successful, the browser will be redirected to the 'admin' page.
	 * @param integer $id the ID of the model to be deleted
	 */
	public function actionDelete($id)
	{
		if(Yii::app()->request->isPostRequest)
		{
			// we only allow deletion via POST request
			$this->loadModel($id)->delete();

			// if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
			if(!isset($_GET['ajax']))
				$this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
		}
		else
			throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
	}

	/**
	 * Lists all models.
	 */
	public function actionIndex()
	{
		/*$dataProvider=new CActiveDataProvider('SipAccounts');
		$this->render('index',array(
			'dataProvider'=>$dataProvider,
		));*/
		
		$this->actionAdmin();
	}

	/**
	 * Manages all models.
	 */
	public function actionAdmin()
	{
		$model=new SipAccounts('search');
		$model->unsetAttributes();  // clear any default values
		if(isset($_GET['SipAccounts'])) {
			$model->attributes=$_GET['SipAccounts'];
                }

                $displayCreateButton = false;
                if (count($this->getUserListData($model))) {
                    $displayCreateButton = true;
                }
		$this->render('admin',array(
			'model'=>$model,
                        "displayCreateButton" => $displayCreateButton
		));
	}

	/**
	 * Returns the data model based on the primary key given in the GET variable.
	 * If the data model is not found, an HTTP exception will be raised.
	 * @param integer the ID of the model to be loaded
	 */
	public function loadModel($id)
	{
		$model=SipAccounts::model()->findByPk($id);
		if($model===null)
			throw new CHttpException(404,'The requested page does not exist.');
		return $model;
	}

	/**
	 * Performs the AJAX validation.
	 * @param CModel the model to be validated
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='sip-accounts-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}
        
        public function actionAjaxSetCreateBtnVisibility()
        {
            $userLisitData = $this->getUserListData();
            
            if (count($userLisitData)) {
                echo json_encode(array('visibility'=>true));
            } else {
                echo json_encode(array('visibility'=>false));
            }
        }
}
