<?php

class DefaultController extends Controller
{
	/**
	 * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
	 * using two-column layout. See 'protected/views/layouts/column2.php'.
	 */
	public $layout='//layouts/column2';

	/**
	 * @return array action filters
	 */
	public function filters()
	{
		return array(
			'accessControl', // perform access control for CRUD operations
		);
	}

	/**
	 * Specifies the access control rules.
	 * This method is used by the 'accessControl' filter.
	 * @return array access control rules
	 */
	public function accessRules()
	{
		return array(
			array('allow',  // allow all users to perform 'index' and 'view' actions
				'actions'=>array('index','view','check'),
				//'users'=>array('@'),
				'expression'=>AccessManager::checkSessionPermission($this->module->name),
			),
			array('allow', // allow authenticated user to perform 'create' and 'update' actions
				'actions'=>array('create','update'),
				//'users'=>array('@'),
				'expression'=>AccessManager::checkSessionPermission($this->module->name),
			),
			array('allow', // allow admin user to perform 'admin' and 'delete' actions
				'actions'=>array('admin','delete'),
				//'users'=>array('@'),
				'expression'=>AccessManager::checkSessionPermission($this->module->name),
			),
			array('deny',  // deny all users
				'users'=>array('*'),
			),
		);
	}

	/**
	 * Displays a particular model.
	 * @param integer $id the ID of the model to be displayed
	 */
	public function actionView($id)
	{
		$this->render('view',array(
			'model'=>$this->loadModel($id),
		));
	}

	/**
	 * Creates a new model.
	 * If creation is successful, the browser will be redirected to the 'view' page.
	 */
	public function actionCreate()
	{
		$model=new EMails;

		// Uncomment the following line if AJAX validation is needed
		// $this->performAjaxValidation($model);

		if(isset($_POST['EMails']))
		{
			$model->attributes=$_POST['EMails'];
			if($model->save())
				$this->redirect(array('view','id'=>$model->id));
		}

		$this->render('create',array(
			'model'=>$model,
		));
	}
	
	public function actionCheck()
	{
		// Uncomment the following line if AJAX validation is needed
		// $this->performAjaxValidation($model);
	
		$imap = new ImapConnector("imap.aerea.de", "user@domain.de", "passwordinplaintext", true, true, false);
		if ($imap->connect()) {
			echo "connected.\n";
		} else {
			echo "error connecting.\n";
			die();
		}
		
		echo "There are ".$imap->messageCount()." Messages on the Server.\n";
		
		echo "These are the imap folders:\n";
		var_dump($imap->getFolders());
		
		//$imap->cd("Gesendet");
		//echo "in Gesendet are: ".$imap->messageCount()." messages.\n";
		//$imap->disconnect();
		$count = $imap->messageCount();
		$model = new EMails;
		$model->deleteAll();
		
		for ($i = 1; $i <= $count; $i++) {
			
			$model = EMailMessageToModelAdapter::fromEMailToModel($imap->getEMailMessageById($i));
			//var_dump($model);
			/*$header = $imap->getMailHeader($i);
			echo "<pre>";
			var_dump($header);
			echo "</pre>";
			$raw_body = $imap->getMailRawBody($i);
			//var_dump($raw_body);
			
			$model = new EMails;
			$model->isNewRecord = TRUE;
			$model->userId = 1;
			$model->timestamp = @$header->udate; 
			$subject = imap_mime_header_decode(@$header->subject);
			$model->subject = $subject[0]->text; 
			
			$convert = imap_mime_header_decode(@$header->from[0]->personal);			
			$model->from_personal = utf8_encode(@$convert[0]->text);
			print_r($convert);
			$model->from_host = @$header->from[0]->host;
			$model->from_mailbox = @$header->from[0]->mailbox;
			
			$model->to_host = @$header->to[0]->host;
			$convert = imap_mime_header_decode(@$header->to[0]->personal);
			print_r($convert);
			$model->to_personal = utf8_encode(@$convert[0]->text);
			$model->to_mailbox = @$header->to[0]->mailbox;
			
			$model->body_raw = $raw_body;
			
			
			*/
			var_dump($model->save(false));
			echo $i."\n";
			
			if($i==100) {
				break;
			}
			flush();
			
			
		}
		
		
		
		
	}
	

	/**
	 * Updates a particular model.
	 * If update is successful, the browser will be redirected to the 'view' page.
	 * @param integer $id the ID of the model to be updated
	 */
	public function actionUpdate($id)
	{
		$model=$this->loadModel($id);

		// Uncomment the following line if AJAX validation is needed
		// $this->performAjaxValidation($model);

		if(isset($_POST['EMails']))
		{
			$model->attributes=$_POST['EMails'];
			if($model->save())
				$this->redirect(array('view','id'=>$model->id));
		}

		$this->render('update',array(
			'model'=>$model,
		));
	}

	/**
	 * Deletes a particular model.
	 * If deletion is successful, the browser will be redirected to the 'admin' page.
	 * @param integer $id the ID of the model to be deleted
	 */
	public function actionDelete($id)
	{
		if(Yii::app()->request->isPostRequest)
		{
			// we only allow deletion via POST request
			$this->loadModel($id)->delete();

			// if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
			if(!isset($_GET['ajax']))
				$this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('admin'));
		}
		else
			throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
	}

	/**
	 * Lists all models.
	 */
	public function actionIndex()
	{
		$dataProvider=new CPhonalisaDataProvider('EMails');
		$this->render('index',array(
			'dataProvider'=>$dataProvider,
		));
	}

	/**
	 * Manages all models.
	 */
	public function actionAdmin()
	{
		$model=new EMails('search');
		$model->unsetAttributes();  // clear any default values
		if(isset($_GET['EMails']))
			$model->attributes=$_GET['EMails'];

		$this->render('admin',array(
			'model'=>$model,
		));
	}

	/**
	 * Returns the data model based on the primary key given in the GET variable.
	 * If the data model is not found, an HTTP exception will be raised.
	 * @param integer the ID of the model to be loaded
	 */
	public function loadModel($id)
	{
		$model=EMails::model()->findByPk($id);
		if($model===null)
			throw new CHttpException(404,'The requested page does not exist.');
		return $model;
	}

	/**
	 * Performs the AJAX validation.
	 * @param CModel the model to be validated
	 */
	protected function performAjaxValidation($model)
	{
		if(isset($_POST['ajax']) && $_POST['ajax']==='emails-form')
		{
			echo CActiveForm::validate($model);
			Yii::app()->end();
		}
	}
}
