<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array(
                    'index',
                    'update',
                    'rebootSystem',
                    'writeNetworkConfig',
                    'writeDHCPSettings'
                ),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function actionIndex()
    {
        $model = NetworkSettings::model();

        PhonalisaCtrldConnector::hasConnection();
        $this->render('networkAdapter', array('model' => $model));
    }

    public function actionUpdate($id)
    {
        $model = $this->loadNetworkInterfaceModel($id);
        $this->performAjaxValidation($model);

        if (isset($_POST['NetworkInterface'])) {
            $model->attributes = $_POST['NetworkInterface'];
            $model->subnet = $this->getSubnetAttribute($model);
            $this->saveModel($model);
        }
        $this->renderPartial('_form', array('model' => $model), false, true);
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadNetworkInterfaceModel($id)
    {
        $model = NetworkInterface::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'network-adapter-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }

    protected function getSubnetAttribute($model)
    {
        $subnet = $model->subnet;
        if (empty($subnet) || !cidr_match($model->ip, $subnet)) {
            $subnet = $model->getIpAsCIDR();
        }
        return $subnet;
    }

    protected function saveModel($model)
    {
        if ($model->save()) {
            Yii::app()->user->setSuccessFlashMessage();
        } else {
            Yii::app()->user->setErrorFlashMessage();
        }
    }

    public function actionRebootSystem()
    {
        $id = '';
        if (isset($_REQUEST['id'])) {
            $id = $_REQUEST['id'];
        }
        PhonalisaCtrldConnector::rebootSystem();
        $url = url('site/reboot', array('redirect' => 'site/logout', 'id' => $id));
        echo json_encode($url);
        Yii::app()->end();
    }

    public function changeSettingsFromPrimaryNetworkAdapter($id)
    {
        $networkInterface = $this->loadNetworkInterfaceModel($id);
        if ($networkInterface) {
            return $networkInterface->changeDhcpWithPrimaryNetworkAdapterSettings();
        }
    }

    public function actionWriteNetworkConfig()
    {
        $result = 'error';
        if (NetworkInterface::writeNetworkConfigsToCtrld()) {
            $result = 'ok';
            app()->user->setSuccessFlashMessage();
        } else {
            app()->user->setErrorFlashMessage('NETWORK_SETTINGS_WRITE_TO_CTRLD_ERROR');
        }
        echo json_encode($result);
        Yii::app()->end();
    }

    public function actionWriteDHCPSettings()
    {
        $result = 'error';
        if ($_REQUEST['id']) {
            if ($this->changeSettingsFromPrimaryNetworkAdapter($_REQUEST['id']) != false) {
                $result = 'ok';
                app()->user->setSuccessFlashMessage();
            }
        }
        if ($result == 'error') {
            app()->user->setErrorFlashMessage('NETWORK_SETTINGS_WRITE_TO_CTRLD_ERROR');
        }
        
        echo json_encode($result);
        Yii::app()->end();
    }
}
