<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'dnsRecord', 'renderHostField'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function actionIndex()
    {
        if (isset($_REQUEST['saveBtn'])) {
            $this->deleteDNSServerModels($this->getSavedDNSRecordIDs('DNSServer'));
            $this->deleteDNSZoneModels($this->getSavedDNSRecordIDs('DNSZone'));
            $this->sendDNSDataToCtrld();

            if (!app()->user->hasFlash('error')) {
                app()->user->setSuccessFlashMessage();
            }
        }
        $models = $this->getDNSServerModels();
        $dataProvider = $this->getDNSArrayDataProvider($models);
        $checboxDataProvider = $this->getDNSArrayDataProvider(array($models[0]));

        $dataProviderZone = $this->getDNSArrayDataProvider($this->getDNSZoneModels());
        $defaultTtlValue = new DNSZone();

        $this->render('dnsManagement', array(
            'dataProvider' => $dataProvider,
            'dataProviderZone' => $dataProviderZone,
            'checboxDataProvider' => $checboxDataProvider,
            'defaultTtlValue' => $defaultTtlValue->ttl
                )
        );
    }

    protected function getSavedDNSRecordIDs($modelName, $dnsZone = '')
    {
        $dnsInputs = @$_REQUEST[$modelName];
        $enableDNSServer = @$dnsInputs['enableDNSServer'];
        $savedDnsIDs = array();
        if (isset($dnsInputs)) {
            foreach ($dnsInputs as $key => $dnsInput) {
                if (!is_array($dnsInput) || (isset($dnsInput['fqdn']) && $dnsInput['fqdn'] == '')) {
                    continue;
                }

                $dnsID = @$dnsInput['dnsID'];
                $dnsModel = '';

                if (!empty($dnsID)) {
                    $dnsModel = $modelName::model()->findByPk($dnsID);
                }
                if (!$dnsModel) {
                    $dnsModel = new $modelName();
                }
                unset($dnsInput['dnsID']);
                $dnsModel->attributes = $dnsInput;
                $dnsModel->position = (string) $key;

                if (isset($enableDNSServer)) {
                    $dnsModel->enableDNSServer = $enableDNSServer;
                }

                if (!empty($dnsZone)) {
                    $dnsModel->dnsZone = $dnsZone;
                }

                if (!$dnsModel->save()) {
                    app()->user->setErrorFlashMessage('CHANGES_NOT_SAVED_CAUSE_OF_AN_ERROR', $dnsModel->getFormatedErrors());
                }
                $savedDnsIDs[$dnsModel->id] = $dnsModel->id;
            }
        }
        return $savedDnsIDs;
    }

    protected function deleteDNSZoneModels($savedDNSRecordIDs)
    {
        $models = DNSZone::model()->findAll();
        foreach ($models as $value) {
            if (!isset($savedDNSRecordIDs[$value->id])) {
                $value->delete();
            }
        }
    }

    protected function sendDNSDataToCtrld()
    {
        $data = DNSServer::getDnsDataForCtrld();
        $ctrld = PhonalisaCtrldConnector::writeDnsConfig($data);
        $this->writeErrorMsgIfCtrldResultHasError($ctrld);
    }

    protected function writeErrorMsgIfCtrldResultHasError($ctrld)
    {
        $ctrldStatus = @$ctrld->status;
        $ctrldErrorTxt = @$ctrld->errorText;

        if (empty($ctrldStatus)) {
            app()->user->setErrorFlashMessage('GLOBAL_WRITE_TO_CTRLD_ERROR');
        } else if ($ctrldStatus == 'error' && !empty($ctrldErrorTxt)) {
            app()->user->setErrorFlashMessage($ctrldErrorTxt);
        }
    }

    protected function getDNSServerModels()
    {
        $models = DNSServer::model()->findAll();
        if (!$models) {
            $models = array(new DNSServer());
        }
        return $models;
    }

    protected function getDNSArrayDataProvider($data)
    {
        $sort = new CSort;
        $sort->defaultOrder = 'position';
        return new CArrayDataProvider($data, array('pagination' => false, 'sort' => $sort));
    }

    protected function deleteDNSServerModels($savedDNSRecordIDs)
    {
        $models = DNSServer::model()->findAll();
        foreach ($models as $value) {
            if (!isset($savedDNSRecordIDs[$value->id])) {
                $value->delete();
            }
        }
    }

    protected function getDNSZoneModels()
    {
        $models = DNSZone::model()->findAll();
        if (!$models) {
            $models = array(new DNSZone());
        }
        return $models;
    }

    public function actionDnsRecord($dnsZone)
    {
        PhonalisaCtrldConnector::hasConnection();
        if (isset($_REQUEST['saveBtn'])) {
            $savedDNSRecordIDs = $this->getSavedDNSRecordIDs('DNSRecord', $dnsZone);
            $this->deleteDNSRecordModels($dnsZone, $savedDNSRecordIDs);
            $this->sendDNSDataToCtrld();

            if (!app()->user->hasFlash('error')) {
                app()->user->setSuccessFlashMessage();
            }
        }
        $models = $this->getDNSRecordModels($dnsZone);
        $this->render('dnsRecords', array(
            'dataProvider' => $this->getDNSArrayDataProvider($models),
            'dnsZone' => $dnsZone,
            'domainName' => $this->getDNSRecordNavbarTitle($dnsZone)
        ));
    }

    protected function getDNSRecordModels($dnsZone)
    {
        $models = DNSRecord::model()->findAllByAttributes(array('dnsZone' => $dnsZone));
        if (!$models) {
            $models = array(new DNSRecord());
        }
        return $models;
    }

    protected function deleteDNSRecordModels($dnsZone, $savedDNSRecordIDs)
    {
        $models = DNSRecord::model()->findAllByAttributes(array('dnsZone' => $dnsZone));
        foreach ($models as $value) {
            if (!isset($savedDNSRecordIDs[$value->id])) {
                $value->delete();
            }
        }
    }

    public function renderInput($data, $row, $column)
    {
        $attribute = $column;
        if (isset($column->name)) {
            $attribute = $column->name;
        }
        $fieldName = get_class($data) . "[$row]";

        $html = CHtml::activeTextField($data, $attribute, array('name' => $fieldName . '[' . $attribute . ']'));
        $html.=CHtml::hiddenField($fieldName . '[dnsID]', $data->id);
        return $html;
    }

    public function renderNumberField($data, $row, $column)
    {
        $fieldName = get_class($data) . "[$row]";
        $html = CHtml::activeNumberField($data, $column->name, array(
                    'name' => $fieldName . '[' . $column->name . ']',
                    'min' => "1"
        ));
        $html.=CHtml::hiddenField($fieldName . '[dnsID]', $data->id);
        return $html;
    }

    public function renderCheckbox($checkboxModel)
    {
        $fieldName = get_class($checkboxModel);
        return CHtml::activeCheckBox(
                        $checkboxModel, 'enableDNSServer', array('name' => $fieldName . '[enableDNSServer]')
        );
    }

    public function renderDNSRecordType($data, $row, $column)
    {
        $fieldName = get_class($data) . "[$row]";
        $list = DNSRecordType::getDNSRecordTypesArray();
        return CHtml::activeDropDownList(
                        $data, $column->name, $list, array('name' => $fieldName . '[' . $column->name . ']')
        );
    }

    public function renderLabel($data, $row, $column)
    {
        return "<div>" . __($column->name) . "</div>";
    }

    public function getDNSRecordNavbarTitle($dnsZone)
    {
        $domainName = '';
        $dnsZoneModel = DNSZone::model()->findByPk($dnsZone);
        if ($dnsZoneModel) {
            $domainName = $dnsZoneModel->fqdn;
        }
        return __('DNS_RECORD_NAVBAR_TITLE', true, 'messages', array("{name}" => $domainName));
    }

    public function actionRenderHostField()
    {
        $hostFieldName = @$_REQUEST['hostFieldName'];
        $dnsZoneID = @$_REQUEST['dnsZoneID'];
        $dnsRecordID = @$_REQUEST['dnsRecordID'];
        $type = @$_REQUEST['type'];
        $hostFieldValue = @$_REQUEST['hostFieldValue'];

        $model = new DNSRecord();
        if (!empty($dnsRecordID)) {
            $model = DNSRecord::model()->findByPk($dnsRecordID);
        }

        if (!empty($dnsZoneID) && ($type === DNSRecordType::MX || $type === DNSRecordType::CNAME)) {
            echo $this->renderHostFieldDropdown($model, $dnsZoneID, $hostFieldName);
        } else {
            echo $this->renderHostFieldInput($model, $hostFieldName, $hostFieldValue);
        }
    }

    protected function renderHostFieldDropdown($model, $dnsZoneID, $hostFieldName)
    {
        $models = DNSRecord::model()->findAllByAttributes(array('type' => DNSRecordType::A, 'dnsZone' => $dnsZoneID));
        $list = array();
        foreach ($models as $listModel) {
            if ($listModel->name != '*' && $listModel->name != '@') {
                $list[$listModel->name] = $listModel->name;
            }
        }
        return CHtml::activeDropDownList($model, 'host', $list, array('name' => $hostFieldName));
    }

    protected function renderHostFieldInput($model, $hostFieldName, $hostFieldValue)
    {
        if (isset($hostFieldValue)) {
            $model->host = $hostFieldValue;
        }
        $row = str_replace(array('DNSRecord[', '][host]'), '', $hostFieldName);
        return $this->renderInput($model, $row, 'host');
    }
}
