<?php

class DNSServer extends PhonalisaRecord
{

    public $enableDNSServer;
    public $upstreamDNSServer;
    public $position;

    public static function myAttributes()
    {
        return array(
            'enableDNSServer' => 'enableDNSServer',
            'upstreamDNSServer' => 'upstreamDNSServer',
            'position' => 'position'
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Job the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('enableDNSServer, upstreamDNSServer, position', 'safe'),
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'enableDNSServer' => __('DNS_SERVER_ENABLE_DNS_SERVER'),
            'upstreamDNSServer' => __('DNS_SERVER_UPSTREAM_DNS_SERVER'),
        );
    }

    public static function getDnsDataForCtrld()
    {
        $data = array();
        $data['domains'] = array();
        $data['enableDNSServer'] = '0';
        $data['upstreamDNSServers'] = array();

        $dnsServers = DNSServer::model()->find(array(), 'position');
        foreach ($dnsServers as $dnsServer) {
            $data['enableDNSServer'] = $dnsServer->enableDNSServer;
            $data['upstreamDNSServers'][] = $dnsServer->upstreamDNSServer;
        }

        $dnsZones = DNSZone::model()->find(array(), 'position');
        foreach ($dnsZones as $dnsZone) {
            $dnsRecords = DNSRecord::model()->find(array('dnsZone' => $dnsZone->id), 'position');
            foreach ($dnsRecords as $key => $dnsRecord) {
                $data['domains'][$dnsZone->fqdn]['records'][$key]['host'] = $dnsRecord->host;
                $data['domains'][$dnsZone->fqdn]['records'][$key]['name'] = $dnsRecord->name;
                if ($dnsRecord->type === DNSRecordType::MX) {
                    $data['domains'][$dnsZone->fqdn]['records'][$key]['prio'] = $dnsRecord->prio;
                }
                $data['domains'][$dnsZone->fqdn]['records'][$key]['type'] = DNSRecordType::getDNSRecordType($dnsRecord->type);
            }
            $data['domains'][$dnsZone->fqdn]['ttl'] = $dnsZone->ttl;
        }

        if ($data['enableDNSServer'] == '0') {
            if (count($data['domains']) == 0) {
                $data['domains']['phonalisa.lan'] = array();
            }
            if (!isset($data['upstreamDNSServers'][0]) || $data['upstreamDNSServers'][0] === '') {
                $data['upstreamDNSServers'][0] = '127.0.0.1';
            }
        }

        return $data;
    }
}
