<?php

abstract class AbstractProviderConnector extends Controller
{

    protected $limit = 25;
    protected $offset = 0;
    protected $progress = 0;

    abstract protected function gatherMetaDataForSyncJob($model);

    abstract protected function initiateProviderSpecificSync($meta, $model);

    public function startSync($thirdPartyCredential)
    {
        if (!$this->hasRunningJob($thirdPartyCredential)) {
            if (!empty($thirdPartyCredential->addressBookId) || !empty($thirdPartyCredential->syncCalendars)) {
                $this->gatherMetaDataForSyncJob($thirdPartyCredential);
            } else {
                $this->showFlashMessages(__('THIRDPARTYCREDENTIAL_NOT_EXISTING_ADDRESS_BOOK'), 'error');
            }
        }
    }

    protected function hasRunningJob($model)
    {
        
        $jobs = Job::model()->findAllByAttributes(array('type' => Job::AddressImport));
        
        foreach ($jobs as $syncJob) {
            $meta = json_decode($syncJob->meta, true);
            if (in_array($model->id, $meta)) {
                Log::d("Job to model $model->id already exists.");
                return true;
            }
        }
        return false;
    }

    protected function createErrorLogMessages($model)
    {
        $class = get_class($model);
        Log::e("An error has occurred saving $class-ID $model->id", "ThirdPartyCredentials[$class]");
        foreach ($model->errors as $key => $errorMsg) {
            Log::e("Error in $key ");
            if (is_array($errorMsg)) {
                Log::e("ErrorMessage in " . array_shift($errorMsg));
            }
        }
    }

    protected function createSyncJob($id, $totalItemCount)
    {
        $job = new Job();
        $job->type = Job::AddressImport;
        $job->user_id = isset(app()->user->sudoUser) ? app()->user->sudoUser : app()->user->realUserId;
        $job->setJsonMeta($this->limit, $this->offset, $id, $totalItemCount, $this->progress);
        if (!$job->save()) {
            $this->createJobErrorLogMessage($job);
        }
    }

    public function synchronizeAccounts($job, $model)
    {
        $meta = null;

        $meta = $this->getJobMeta($job);
        if ($meta->progress < 100) {
            Log::d("Sync-Status: $meta->offset of $meta->totalItemCount", 'ThirdPartyCredentials');
            $newOffset = $this->initiateProviderSpecificSync($meta, $model);
        }
        if ($newOffset < $meta->totalItemCount) {
            $progress = (string) floor(($newOffset / $meta->totalItemCount) * 100);
            $job->setJsonMeta($meta->limit, $newOffset, $meta->thirdPartyCredentialId, $meta->totalItemCount, $progress);
        } else {
            $meta->offset = $meta->totalItemCount;
            $meta->progress = 100;
            Log::v("Sync-Status: Finished", 'ThirdPartyCredentials');
        }
        if (!$job->save()) {
            $this->createJobErrorLogMessage($job);
        }
        echo json_encode($meta);
    }

    protected function getJobMeta($job)
    {
        if ($job) {
            return $job->getJsonMeta();
        } else {
            throw new CHttpException(404, 'Job model not found.');
        }
    }

    protected function createJobErrorLogMessage($job)
    {
        Log::e("An error has occurred in Job:  ID $job->id", 'phonalisa-web-ThirdPartyCredential');
        Log::e("User-ID: $job->user_id", 'phonalisa-web-ThirdPartyCredential');
        Log::e('Status: ' . $job->status, 'ThirdPartyCredentials');
        Log::e('Error Text : ' . json_encode($job->errors), 'ThirdPartyCredentials');
    }

    protected function showFlashMessages($message, $type)
    {
        Yii::app()->user->setFlash($type, $this->setFlashMessages($message));
        Yii::app()->request->redirect(url('/phonesettingsThirdPartyCredentials/default/index'));
    }

    protected function setFlashMessages($message)
    {
        return "<strong>$message</strong><br/>";
    }
}
