<?php

include dirname(__FILE__) . '/../../../lib/AbstractXmlPhoneConfigApp.php';
require_once "lib/XmlMenuScreen.php";
require_once "lib/XmlPhoneDirectoryScreen.php";
require_once "lib/XmlInputScreen.php";
require_once "lib/XmlButtons.php";
require_once "lib/XmlTextScreen.php";
require_once "lib/XmlExit.php";

class XmlPhoneConfigApp extends AbstractXMLPhoneConfigApp
{

    public function renderConfigMenuList($menuentries)
    {
        $this->setConfigAppLanguage();
        $URI = $this->getMainMenuItemURI();

        $screen = new XmlMenuScreen();
        $screen->setTitle(__("Konfiguration"));

        $select = "true";
        foreach ($menuentries as $menuentry) {
            $screen->addMenuItem($menuentry["title"], $URI . "?section=" . $menuentry["url"], $select);
            $select = "false";
        }

        $screen->addButton(XmlIndexButtons::F1, __("XML_GLOBAL_EXIT_BTN"), $this->getExitURL());
        $screen->addButton(XmlIndexButtons::F4, __("XML_GLOBAL_SELECT_BTN"), "", XmlSoftkeyButtons::ENTER);

        $screen->fetchUrlAfterMilSeconds($this->getExitURL());
        $screen->printScreen();
    }

    public function renderCallForwardingsList($list, $activeID)
    {
        $this->setConfigAppLanguage();
        if (empty($list)) {
            return $this->renderError('', __('XML_GLOBAL_NO_ENTRIES_FOUND'));
        }

        $URI = $this->getForwardMenuURI();
        $cnt = 1;
        foreach ($list as $model) {
            if ($model->id === $activeID) {
                break;
            }
            ++$cnt;
        }
        $moveTo = $URI . "&editMode=1";
        $screen = new XmlMenuScreen();
        $screen->setTitle(__("XML_CONFIG_APP_CALL_FORWARDING_TITLE"));

        foreach ($list as $key => $model) {
            $text = '';
            $select = 'false';
            if ($key == $cnt) {
                $select = "true";
                $moveTo = "$URI&k=$key&setItem=$model->id&editMode=1";
                $text.='*';
            }
            $text.= $model->name;
            $target = $model->getSimpleTargetNumber();
            if ($target != "") {
                $text .= " (" . $target . ")";
            }
            
            
            $screen->addMenuItem($text, "$URI&k=$key&setItem=$model->id", $select);
        }
        $screen->addButton(XmlIndexButtons::F3, __("XML_CONFIG_APP_EDIT_BTN"), $moveTo);
        if ($this->getPhoneModel()->type != 'snom870') {
            $screen->addButton(XmlIndexButtons::F1, __("XML_GLOBAL_EXIT_BTN"), $this->getExitURL());
            $screen->addButton(XmlIndexButtons::F4, __("XML_CONFIG_APP_APPLY_SETTINGS"), "", XmlSoftkeyButtons::ENTER);
        }

        $screen->fetchUrlAfterMilSeconds($this->getExitURL());
        $screen->printScreen();
    }

    public function renderCallForwardingItemEditTargetNumberScreen($number, $itemId)
    {
        $this->setConfigAppLanguage();
        $URI = $this->getForwardMenuURI();

        $screen = new XmlInputScreen();
        $screen->setPrompt(__('XML_CONFIG_APP_CALL_DESTINATION_TITLE'));

        $screen->setURl("$URI&saveItem=$itemId&setItem=$itemId");
        $screen->setInputItem(__('XML_CONFIG_APP_CALL_DESTINATION_INPUT_TEXT'), 'number', XmlInputFlags::PHONE_NUMBER, $number);

        $screen->addButton(XmlIndexButtons::F1, __("XML_GLOBAL_BACK_BTN"), $URI);
        $screen->addButton(XmlIndexButtons::F4, __("XML_CONFIG_APP_SAVE_BTN"), "", XmlSoftkeyButtons::ENTER);

        $screen->fetchUrlAfterMilSeconds($this->getExitURL());
        $screen->printScreen();
    }

    public function renderError($title, $text)
    {
        $this->setConfigAppLanguage();
        $screen = new XmlTextScreen();
        $screen->setTitle($title);
        $screen->setText($text);
        $screen->addButton(XmlIndexButtons::F1, __("XML_CONFIG_APP_CONFIRM_BTN"), $this->getMainMenuItemURI());
        $screen->fetchUrlAfterMilSeconds($this->getExitURL());
        $screen->printScreen();
    }

    public function renderExit()
    {
        $screen = new XmlExit();
        $screen->printScreen();
    }
}
