<?php

require_once dirname(__FILE__) . '/../tiptel-2xx/Tiptel.php';

class Tiptel32xx extends Tiptel
{

    public function __construct($manufactorName = "Tiptel32xx", $phoneModelName = "", $phoneUserAgent = "")
    {
        return parent::__construct($manufactorName, $phoneModelName, $phoneUserAgent);
    }

    public static function getManufactorName()
    {
        return "Tiptel32xx";
    }

    public static function getFunctionPlaceholders()
    {
        return array(
            'no' => 'PHONE_TIPTEL_FUNCTION_NOT_OVERRIDE',
            '0' => 'PHONE_TIPTEL_FUNCTION_EMPTY',
            '10' => 'PHONE_TIPTEL_FUNCTION_SPEEDDIAL', // SpeedDial
            '13' => 'PHONE_TIPTEL_FUNCTION_REDIAL', // Redial
            '14' => 'PHONE_TIPTEL_FUNCTION_LINE', // Line (for line key only)
            '16' => 'PHONE_TIPTEL_FUNCTION_BUSY_LAMP_FIELD', // BLF
            '20' => 'PHONE_TIPTEL_FUNCTION_RECORD', // Record
            'X01' => 'PHONE_TIPTEL_FUNCTION_FORWARD', // Call Forward
            'X02' => 'PHONE_TIPTEL_FUNCTION_NIGHT_CIRCUT', // Night Circuit
            'X03' => 'PHONE_TIPTEL_FUNCTION_QUEUE_LOGIN', // Queue Login/Logout
            'X04' => 'PHONE_TIPTEL_FUNCTION_SPEEDDIAL_PHONALISA', // SpeedDial
            'X05' => 'PHONE_TIPTEL_FUNCTION_VOICEMAIL', // Voicemail
            'X06' => 'PHONE_TIPTEL_FUNCTION_CALL_PICKUP', // Direct Pickup
            'X07' => 'PHONE_TIPTEL_FUNCTION_INTERCOM', // Intercom
            'X08' => 'PHONE_TIPTEL_FUNCTION_DND', // DND
            'X09' => 'PHONE_TIPTEL_FUNCTION_LISTENING', // Group Listening
            'X10' => 'PHONE_TIPTEL_FUNCTION_ALL_QUEUES_LOGOUT', // Logout from all Queues
            'X11' => 'PHONE_TIPTEL_FUNCTION_HOT_DESKING', // Hot Desking
            'X12' => 'PHONE_TIPTEL_FUNCTION_ALL_QUEUES_LOGIN', // Login to all Queues
        );
    }

    public static function getFunctions($translated = true, $phone = '', $level = '')
    {
        $placeholders = self::getFunctionPlaceholders();

        if ($translated) {
            $functions = array();
            foreach ($placeholders as $key => $value) {
                $functions[$key] = __($value);
            }
            return $functions;
        } else {
            return $placeholders;
        }
    }

    public function getAvailableKeys($level)
    {
        $phones = $this->getSupportedPhoneModels();
        if (!array_key_exists($this->mPhoneModelName, $phones)) {
            return array();
        }

        $phone = $phones[$this->mPhoneModelName];
        $btnData = array();

        if ($level == "" || $level == "line") {
            $btnData = $this->getBtnData($btnData, $phone->getLineKeys(), 'line');
        }

        if ($level == "" || $level == "memory") {
            $btnData = $this->getBtnData($btnData, $phone->getKeys(), 'memory');
        }

        $extKeys = $phone->getExtKeys();
        $extModules = $phone->getExtModules();

        for ($i = 0; $i <= $extModules - 1; $i++) {
            if ($level == "" || $level == 'ext' . $i) {
                for ($j = 1; $j <= $extKeys; $j++) {
                    $button = new PhoneKey();
                    $button->phone = $this->mPhoneModelName;
                    $button->function = '';
                    $button->level = 'ext' . $i;
                    $button->key_idx = "ext" . $i . '.' . $j;
                    $button->name = "Ext " . ($i + 1) . '.' . $j;
                    $button->availableFunctions = $this->getFunctions(true, $button->phone, $button->level);
                    $btnData[$button->name] = $button;
                }
            }
        }

        return $btnData;
    }

    public function getBtnData($btnData, $phoneKeys, $level)
    {
        for ($i = 1; $i <= $phoneKeys; $i++) {
            $button = new PhoneKey();
            $button->name = ucfirst($level) . " $i";
            $button->phone = $this->mPhoneModelName;
            $button->function = '';
            $button->level = $level;
            $button->key_idx = $i;
            $button->availableFunctions = $this->getFunctions(true, $button->phone, $button->level);
            $btnData[$button->name] = $button;
        }
        return $btnData;
    }

    public static function getSupportedPhoneModels()
    {
        $models = array();
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3210');
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3220', 2);
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3220XL', 0, 3);
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3230', 3);
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3235', 8, 0, 1, 20);
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3240', 8, 10, 1, 20);
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3245', 10, 10, 1, 20);
        $models = Tiptel32xx::writePhoneDataToTiptel32xxFactory($models, '3275');
        return $models;
    }

    protected static function writePhoneDataToTiptel32xxFactory($models, $name, $lineKeys = 0, $memoryKeys = 0, $extModules = 0, $extModulesKeys = 0)
    {
        $tmp = new Tiptel32xx(Tiptel32xx::getManufactorName(), "tiptel-" . $name, "tiptel" . $name);
        $tmp->setPhoneModelNameHumanReadable("tiptel " . $name);
        $tmp->setLineKeys($lineKeys);
        $tmp->setKeys($memoryKeys);
        $tmp->setExtModules($extModules);
        $tmp->setExtKeys($extModulesKeys);
        $models[$tmp->getPhoneModelName()] = $tmp;
        return $models;
    }

    public function validateFirmwareFile($filename, $fileTmpPath, &$errorText)
    {
        $firmwareKey = $filename;
        if (endsWith($filename, '.rom')) {
            $firmwareKey = substr($filename, 0, strlen($filename) - 4); // remove .rom
        }

        $parts = explode(".", $firmwareKey);

        if (count($parts) != 4) {
            $errorText = __("FIRMWARE_VALIDATION_ERROR_WRONG_FILE_IS_NOT_VALID");
            return false;
        }

        if (!$this->versionMatchesForPhone($parts[0])) {
            $errorText = __("FIRMWARE_VALIDATION_ERROR_WRONG_FILE_IS_NOT_A_FIRMWARE_FOR") . ": " . $this->mPhoneModelNameHuman;
            return false;
        }

        return Tiptel32xx::tiptel_normalize_version($firmwareKey);
    }

    protected function versionMatchesForPhone($firmwareKey)
    {
        $versionMatchesForPhone = false;
        $phoneName = $this->getPhoneModelName();
        if ($firmwareKey == "15" && $phoneName == "tiptel-3220XL") {
            $versionMatchesForPhone = true;
        } else if ($firmwareKey == "47" && $phoneName == "tiptel-3275") {
            $versionMatchesForPhone = true;
        } else if ($firmwareKey == "50" && $phoneName == "tiptel-3210") {
            $versionMatchesForPhone = true;
        } else if ($firmwareKey == "52" && $phoneName == "tiptel-3220") {
            $versionMatchesForPhone = true;
        } else if ($firmwareKey == "53" && $phoneName == "tiptel-3230") {
            $versionMatchesForPhone = true;
        } else if ($firmwareKey == "59" && $phoneName == "tiptel-3240") {
            $versionMatchesForPhone = true;
        } else if ($firmwareKey == "63" && $phoneName == "tiptel-3235") {
            $versionMatchesForPhone = true;
        } else if ($firmwareKey == "67" && $phoneName == "tiptel-3245") {
            $versionMatchesForPhone = true;
        }
        return $versionMatchesForPhone;
    }

    public static function tiptel_normalize_version($fwvers)
    {
        $tmp = explode('.', $fwvers);
        $v0 = str_pad((int) @$tmp[0], 2, '0', STR_PAD_LEFT);
        $v1 = str_pad((int) @$tmp[1], 2, '0', STR_PAD_LEFT);
        $v2 = str_pad((int) @$tmp[2], 2, '0', STR_PAD_LEFT);
        $v3 = str_pad((int) @$tmp[3], 2, '0', STR_PAD_LEFT);
        return "$v0.$v1.$v2.$v3";
    }
}
