<?php

include dirname(__FILE__) . '/../../../lib/AbstractXmlPhoneConfigApp.php';
require_once "lib/XmlMenuScreen.php";
require_once "lib/XmlInputScreen.php";
require_once "lib/XmlFormattedTextScreen.php";

class XmlPhoneConfigApp extends AbstractXMLPhoneConfigApp
{

    private $screen = null;

    private function getScreenAttributes($changeAttribute = array())
    {
        $screenAttributes = array();
        $screenAttributes["defaultIndex"] = "";
        $screenAttributes["type"] = "number";
        $screenAttributes["editable"] = "yes";
        $screenAttributes["beep"] = "no";
        $screenAttributes["cancelAction"] = "";
        $screenAttributes["wrapList"] = "yes";
        $screenAttributes["Timeout"] = "30";
        $screenAttributes["LockIn"] = "no";
        $screenAttributes["style"] = "number";
        $screenAttributes["InputType"] = '123';

        if (!empty($changeAttribute)) {
            foreach ($changeAttribute as $key => $value) {
                $screenAttributes[$key] = $value;
            }
        }
        return $screenAttributes;
    }

    public function renderConfigMenuList($menuentries)
    {
        $URI = $this->getMainMenuItemURI();
        $screenAttributes = $this->getScreenAttributes();

        $screen = new XmlMenuScreen($screenAttributes);

        $screen->setTitle(__("Konfiguration"));

        foreach ($menuentries as $menuentry) {
            $screen->addMenuItem($menuentry["title"], $URI, "section=" . $menuentry["url"]);
        }

        $screen->addButton("4", "Select");

        $screen->printScreen();
    }

    public function renderCallForwardingsList($list, $activeID)
    {
        if (empty($list)) {
            return $this->renderError('', __('XML_GLOBAL_NO_ENTRIES_FOUND'));
        }

        $URI = $this->getForwardMenuURI();
        $cnt = 1;
        foreach ($list as $model) {
            if ($model->id === $activeID) {
                break;
            }
            ++$cnt;
        }

        $attributes = array(
            'defaultIndex' => $cnt,
            'Previous' => $URI . "&setItem=$activeID&editMode=1",
            'Selection' => $URI
        );

        $screenAttributes = $this->getScreenAttributes($attributes);

        $screen = new XmlMenuScreen($screenAttributes);

        $screen->setTitle(__("XML_CONFIG_APP_CALL_FORWARDING_TITLE"));

        foreach ($list as $key => $model) {
            //print_r($model);
            $text = $model->name;
            $target = $model->getSimpleTargetNumber();
            if ($target != "") {
                $text .= " (" . $target . ")";
            }

            $screen->addMenuItem($text, $URI, "k=" . $key . "&setItem=" . $model->id);
        }

        $screen->addButton("1", __('XML_GLOBAL_EXIT_BTN'), "SoftKey:Cancel");
        $screen->addButton("3", __('XML_CONFIG_APP_EDIT_BTN'), "SoftKey:Previous");
        $screen->addButton("4", __('XML_CONFIG_APP_APPLY_SETTINGS'));

        $screen->printScreen($this->getPhoneModel()->type);
    }

    public function renderCallForwardingItemEditTargetNumberScreen($number, $itemId)
    {
        // render input field for number
        $URI = $this->getForwardMenuURI();
        $attributes = array(
            'type' => 'number',
            'Password' => 'yes',
            'displayMode' => 'normal',
            'Previous' => $this->getMainMenuItemURI()
        );
        $screenAttributes = $this->getScreenAttributes($attributes);

        $screen = new XmlInputScreen($screenAttributes);
        $screen->setTitle(__('XML_CONFIG_APP_CALL_DESTINATION_TITLE'));

        $screen->xw->writeElement("URL", $URI . "&saveItem=" . $itemId . "&setItem=" . $itemId);
        $screen->addInputField("number", "no", "yes", __('XML_CONFIG_APP_CALL_DESTINATION_INPUT_TEXT') . ":", "number", "", $number);

        $screen->addButton("1", __("XML_GLOBAL_BACK_BTN"), "SoftKey:Previous");
        $screen->addButton("3", __('XML_GLOBAL_DELETE_BTN'), "SoftKey:BackSpace");
        $screen->addButton("4", __('XML_CONFIG_APP_SAVE_BTN'), "SoftKey:Submit");

        $screen->printScreen($this->getPhoneModel()->type);
    }

    public function renderError($title, $text)
    {
        $attributes = array(
            'Beep' => 'yes'
        );
        $screenAttributes = $this->getScreenAttributes($attributes);

        $screen = new XmlFormattedTextScreen($screenAttributes);
        $screen->setTitle($title);
        $screen->addText($text);

        $screen->addButton("1", __('XML_CONFIG_APP_CONFIRM_BTN'), "SoftKey:Back");
        $screen->printScreen();
    }
}
