<?php

require_once dirname(__FILE__) . '/../../lib/phoneprovisioning.php';
require_once "Yealink_t4x.php";
require_once "TimezoneHelper.php";

class YealinkT4XPhoneProvisioning extends VirtualPhoneProvisioningController
{

    protected function checkMacAddress($mac)
    {
        return parent::checkMacAddress($mac);
    }

    function startsWith($haystack, $needle)
    {
        return $needle === "" || strpos($haystack, $needle) === 0;
    }

    protected function checkUserAgent($ua)
    {
        //Yealink SIP-T46G 28.71.0.180 00:15:65:45:70:15		
        $ua = str_replace("  ", " ", $ua);

        Log::d("requested Provisioning " . $ua, "yealink-t4x-dls");

        $ua_parts = explode(' ', $ua);
        if (strToLower(@$ua_parts[0]) === 'yealink') {

            $found = false;
            $ua_model = strToLower(@$ua_parts[1]);

            foreach ($this->_getSupportedPhoneModels() as $model) {
                if ($model->getUserAgent() == $ua_model) {
                    $this->setCurrentPhoneModel($model);
                    $found = true;
                    break;
                }
            }

            # find out the firmware version of the phone
            $fw_vers = (preg_match('/(\d+\.\d+\.\d+\.\d+)/', @$ua_parts[2], $m)) ? $m[1] : '0.0.0.0';
            $fw_vers_nrml = $this->normalizeFirmwareVersion($fw_vers);
            $this->setCurrentFirmwareVersion($fw_vers_nrml);
            return $found;
        }
        return parent::checkUserAgent($ua);
    }

    protected function runFirmwareUpdate(Firmware $fwUpdate)
    {
        $port = "";
        if (cfg('PROV_PORT') != "") {
            $port = ":" . cfg('PROV_PORT');
        }
        $this->psetting('firmware.url', "http://" . cfg('PROV_HOST') . $port . "/firmware/" . $fwUpdate->fileName);

        return false;
    }

    protected function loadConfigValues($mac = NULL)
    {
        $template = $this->getTemplate();
        $Phone = $this->getPhoneModel();

        $SipAccount = $Phone->getSipAccount();

        $User = $SipAccount->getUser();
        $callwaiting = false;
        $buttonsound = false;
        if ($User && $User->call_waiting) {
            $callwaiting = true;
        }

        if ($User && $User->key_tone) {
            $buttonsound = true;
        }

        foreach ($template as $key => $value) {
            $this->psetting($key, $value);
        }

        $this->psetting('account.1.enable', '1');
        $this->psetting('account.1.sip_server_host', $SipAccount->getSIPRegistrar());
        $this->psetting('account.1.sip_server_port', cfg("INTERNAL_SIP_PORT"));
        $this->psetting('account.1.user_name', $SipAccount->authname);
        $this->psetting('account.1.auth_name', $SipAccount->authname);
        $this->psetting('account.1.password', $SipAccount->secret);

        $this->psetting('account.1.sip_server.1.address', $SipAccount->getSIPRegistrar());
        $this->psetting('account.1.sip_server.1.port', cfg("INTERNAL_SIP_PORT"));
        $this->psetting('account.1.sip_listen_port', cfg("INTERNAL_SIP_PORT"));

        $label = $SipAccount->getDisplayName();
        $this->psetting('features.config_dsskey_length', '0');
        // check if user has callforwarding -> Change displayname!
        if ($User) {
            if ($User->defaultCallForwardSet != $User->callForwardSet) {
                $this->psetting('features.config_dsskey_length', '1');
                $preset = ForwardPreset::model()->findByPk($User->callForwardSet);
                $closePowerLight = true;
                if ($preset != NULL) {
                    $label = $preset->name;
                } else {
                    $label = "FWD?";
                }
            }
        }

        $this->psetting('account.1.label', $label);
        $this->psetting('account.1.display_name', $SipAccount->getDisplayName());
        $this->psetting('auto_provision.server.url', $this->getProvURL());
        $this->psetting('features.action_uri_limit_ip', cfg('PROV_HOST'));

        $srtp = "0";
        if (cfg("RTP_ENABLE_SRTP") == "1") {
            $srtp = "2";
        }

        $this->psetting('account.1.srtp_encryption', $srtp);
        $this->psetting('account.1.subscribe_mwi_expires', '3600');
        $this->psetting('account.1.sip_server.1.expires', $SipAccount->expires);
        $this->psetting('account.1.cid_source', '2');
        $this->psetting('voice_mail.number.1', 'voicemail');

        $tz = cfg("TIMEZONE");
        if ($User && $User->user_timezone != "") {
            $tz = $User->user_timezone;
        }

        $tz_offset = get_timezone_offset("UTC", $tz, true);
        if ($tz_offset > 0) {
            $tz_offset = "+" . $tz_offset;
        } else if ($tz_offset < 0) {
            $tz_offset = "" . $tz_offset;
        }

        Log::d("Timezone offset: " . $tz_offset . " (" . cfg("TIMEZONE") . ")", "yealink-t4x-dls");
        $this->psetting('local_time.time_zone', $tz_offset); # -11 to +23
        $this->psetting('local_time.ntp_server1', cfg("NTP_HOST")); # NTP Server 1

        $this->psetting('local_time.time_zone_name', 'Germany(Berlin)'); // Location-> needed for Daylight-Saving-Time

        $time_format = cfg("USER_TIME_FORMAT");
        if ($User && $User->user_time_format != "") {
            $time_format = $User->user_time_format;
        }

        if ($time_format == "24") {
            $this->psetting('local_time.time_format', "1");
        } else {
            $this->psetting('local_time.time_format', "0");
        }

        $this->psetting('call_waiting.enable', ($callwaiting ? '1' : '0'));
        $this->psetting('features.key_tone', ($buttonsound ? '1' : '0'));

        $this->psetting('features.bluetooth_enable', '1');

        $language = cfg("GUI_LANGS");
        if ($User && $User->user_language != "") {
            $language = $User->user_language;
        }

        $this->psetting("lang.gui", $this->convertLanguageCode($language));
        $this->psetting("lang.wui", $this->convertLanguageCode($language));

        if ($User) {
            $this->psetting('ldap.enable', '1');
            $this->psetting('ldap.port', '12389');
            $this->psetting('ldap.host', $SipAccount->getSIPRegistrar());
            $this->psetting('ldap.user', 'cn=' . $User->name . ",dc=default");
            $this->psetting('ldap.password', '439rz9f4uifh8vsfdpugh4');
            $this->psetting('ldap.version', '3');
            $this->psetting('ldap.base', 'ou=addressbooks,o=Company,dc=default');
            $this->psetting('ldap.name_attr', 'cn sn displayName');
            $this->psetting('ldap.numb_attr', 'mobile telphoneNumber privatNumber');
            $this->psetting('ldap.name_filter', '(|(cn=%)(sn=%))');
            $this->psetting('ldap.number_filter', '*');
            $this->psetting('ldap.display_name', '%cn');
            $this->psetting('ldap.ldap_sort', '1');
        }

        $port = "";
        if (cfg('PROV_PORT') != "") {
            $port = ":" . cfg('PROV_PORT');
        }
        
        $wallpaper = null;
        if ($this->getPhoneModel()->type === 'yealink-t46g' || $this->getPhoneModel()->type === 'yealink-t46s') {
            $wallpaper = "bg_t46g.png";
        } else if ($this->getPhoneModel()->type === 'yealink-t48g' || $this->getPhoneModel()->type === 'yealink-t48s') {
            $wallpaper = "bg_t48g.png";
        }

        if ($wallpaper != null) {
            $this->psetting('wallpaper_upload.url', "http://" . cfg('PROV_HOST') . $port . "/pictures/".$wallpaper);
            $this->psetting('phone_setting.backgrounds', "Config:".$wallpaper);
        }
        
        // %NULL% = Yealink/Tiptel need this to set the key to default
        $this->psetting('programablekey.2.label', '%NULL%');
        $this->psetting('programablekey.2.value', '%NULL%');
        $this->psetting('programablekey.2.type', '%NULL%');
        $this->psetting('programablekey.3.label', '%NULL%');
        $this->psetting('programablekey.3.value', '%NULL%');
        $this->psetting('programablekey.3.type', '%NULL%');
        $this->psetting('programablekey.14.type', '2');
        $this->psetting('programablekey.14.value', '%NULL%');

        if (!$User) {
            // Login App
            $this->psetting('programablekey.3.type', '27');
            $this->psetting('programablekey.3.value', $this->getProvURL() . "login/" . $Phone->mac_addr . ".xml");
            $this->psetting('programablekey.3.label', 'Login');
        }

        if ($User) {
            // XML-Phonebook
            $this->psetting('programablekey.2.type', '27');
            $this->psetting('programablekey.2.value', $this->getProvURL() . "pb/" . $Phone->mac_addr . ".xml");
            $this->psetting('programablekey.2.label', $this->getXmlPhonebookKeyName($language));

            // XML-Voicemail-Screen
            $this->psetting('programablekey.3.type', '27');
            $this->psetting('programablekey.3.value', $this->getProvURL() . "voicemail/" . $Phone->mac_addr . ".xml");
            $this->psetting('programablekey.3.label', $this->getXmlVoicemailKeyName($language));

            // XML-ConfigApp => Transfer key
            $this->psetting('programablekey.14.type', '27');
            $this->psetting('programablekey.14.value', $this->getProvURL() . "cfgapp/" . $Phone->mac_addr . ".xml?section=forwarding");
        }

        $this->setAudioCodecs();

        $this->processSoftkeys();

        $provisioningProfile = $this->getProvisioningProfile($User);

        if ($provisioningProfile) {
            $params = $provisioningProfile->getParamsForPhone($Phone->type);
            $paramKeys = array_keys($this->mPhonalisaProvisioningVariables);
            foreach ($params as $param) {
                $param->value = str_replace($paramKeys, $this->mPhonalisaProvisioningVariables, $param->value);

                $this->psetting($param->name, $param->value);
                Log::v("Overwriting key from parameter profile: " . $param->name . " = " . $param->value, "tiptel-dls");
            }
        }
        return true;
    }

    protected function processSoftkeys()
    {
        $Phone = $this->getPhoneModel();
        $SipAccount = $Phone->getSipAccount();
        $User = $SipAccount->getUser();

        $levels = array("line", "expansion_module.1", "expansion_module.2", "expansion_module.3");

        foreach ($levels as $level) {

            $softKeys = $Phone->getCurrentPhoneKeys($User, $level);

            foreach ($softKeys as $softKey) {
                $this->processSoftKey($softKey);
            }
        }
    }

    protected function processSoftKey(PhoneKey $key)
    {
        if ($key->function === 'no') {
            return;
        }

        $key->function = $this->setKeyFunctionForOldKeys($key->function);
        $keyFunction = $this->convertSoftKeyFunction($key->function, 16);
        $keyValue = $this->convertSoftKeyValue($key);
        $keyLabel = $key->label;
        $pickupCode = $this->convertPickupCode($key->function, array(16, 'X06'));
        $keyName = $key->name;
        if (empty($key->function)) {
            if ($keyName == "linekey.1") {
                $this->psetting($keyName . '.line', '0');
                $this->psetting($keyName . '.value', '0');
                $this->psetting($keyName . '.type', '15');
                $this->psetting($keyName . '.label', '%NULL%');
            } else {
                $this->psetting($keyName . '.line', '');
                $this->psetting($keyName . '.value', '%NULL%');
                $this->psetting($keyName . '.type', '0');
                $this->psetting($keyName . '.label', '%NULL%');
            }
            return;
        }

        $this->psetting($keyName . '.line', '0');
        $this->psetting($keyName . '.value', $keyValue);
        $this->psetting($keyName . '.type', $keyFunction);
        $this->psetting($keyName . '.label', $keyLabel);

        if ($key->function == 15) {
            $this->psetting($keyName . '.line', $keyValue);
            $this->psetting($keyName . '.value', '%NULL%');
            $this->psetting($keyName . '.label', '%NULL%');
        }

        $this->psetting($keyName . '.pickup_value', $pickupCode);
    }

    protected function setKeyFunctionForOldKeys($keyFunction)
    {
        if ($keyFunction == 5) {
            $keyFunction = 'X08';
        } else if ($keyFunction == 12) {
            $keyFunction = 'X05';
        } else if ($keyFunction == 14) {
            $keyFunction = 'X07';
        }
        return $keyFunction;
    }

    protected function generateOutput()
    {
        echo "#!version:1.0.0.1\n\n";
        ksort($this->mCurrentSettings);
        foreach ($this->mCurrentSettings as $key => $value) {
            Log::d($key . ' = ' . $value, "yealink-t46g-dls");
            echo $key . ' = ' . $value, "\n";
        }

        return false;
    }

    protected function _getSupportedPhoneModels()
    {
        return Yealink_t4x::getSupportedPhoneModels();
    }

    private function normalizeFirmwareVersion($fwvers)
    {
        $tmp = explode('.', $fwvers);
        $v0 = str_pad((int) @$tmp[0], 2, '0', STR_PAD_LEFT);
        $v1 = str_pad((int) @$tmp[1], 2, '0', STR_PAD_LEFT);
        $v2 = str_pad((int) @$tmp[2], 2, '0', STR_PAD_LEFT);
        $v3 = str_pad((int) @$tmp[3], 3, '0', STR_PAD_LEFT);
        return $v0 . '.' . $v1 . '.' . $v2 . '.' . $v3;
    }

    private function getTemplate()
    {
        $contents = file_get_contents(dirname(__FILE__) . "/templates/" . $this->mCurrentPhoneModel->getPhoneModelName() . ".tpl");

        $contents = explode("\n", $contents);
        $config = array();
        foreach ($contents as $line) {

            $line = trim($line);

            if (strlen($line) == 0) {
                continue;
            }

            if ($line[0] == "#") {
                continue;
            }

            $parts = explode("=", $line);

            $key = trim($parts[0]);
            $value = trim(@$parts[1]);

            $config[$key] = $value;
        }
        return $config;
    }

    protected function psetting($sectionname, $val = '')
    {
        $this->mCurrentSettings[$sectionname] = $val;
    }

    protected function getXmlPhonebookKeyName($language)
    {
        $keyName = "Dir.";
        if ($language === "de_de") {
            $keyName = "Verz.";
        }
        return $keyName;
    }

    protected function getXmlVoicemailKeyName($language)
    {
        $keyName = "VMail";
        if ($language === "de_de") {
            $keyName = "AB";
        }
        return $keyName;
    }

    protected function getAccountCount()
    {
        $accounts = 16;
        if ($this->getPhoneModel()->type === 'yealink-t23p') {
            $accounts = 3;
        }
        if ($this->getPhoneModel()->type === 'yealink-t42g') {
            $accounts = 12;
        }
        return $accounts;
    }

    protected function setAudioCodecs()
    {
        $phonalisaCodecs = str_replace(' ', '', cfg('GLOBAL_CODEC_PREFS'));
        $codecs = explode(',', $phonalisaCodecs);
        $phoneCodecs = $this->getPhoneCodecs();
        $accountCount = $this->getAccountCount();

        foreach ($codecs as $key => $value) {
            if (array_key_exists($value, $phoneCodecs)) {
                $phoneCodecs[$value]['enable'] = 1;
                $oldPriority = $phoneCodecs[$value]['priority'];
                $oldCodecWithPriority = $this->searchForPriority($key, $phoneCodecs);
                $phoneCodecs[$oldCodecWithPriority]['priority'] = $oldPriority;
                $phoneCodecs[$value]['priority'] = $key;
            }
        }

        for ($i = 1; $i <= (int) $accountCount; $i++) {
            $this->setAudioCodecToSettings($phoneCodecs, $i);
        }
    }

    protected function getPhoneCodecs()
    {
        $codecs = array(
            'PCMU' => 0,
            'PCMA' => 8,
            'G723_53' => 4,
            'G723_63' => 4,
            'G729' => 18,
            'G722' => 9,
            'iLBC' => 106,
            'G726-16' => 103,
            'G726-24' => 104,
            'G726-32' => 102,
            'G726-40' => 105,
            'GSM' => 3
        );
        $result = array();
        $priority = 0;
        foreach ($codecs as $codec => $rtpmap) {
            $result[$codec] = array(
                'enable' => 0,
                'PayloadType' => $codec,
                'priority' => $priority,
                'rtpmap' => $rtpmap,
            );
            $priority++;
        }
        return $result;
    }

    protected function searchForPriority($id, $array)
    {
        foreach ($array as $key => $val) {
            if ($val['priority'] === $id) {
                return $key;
            }
        }
        return null;
    }

    protected function setAudioCodecToSettings($phoneCodecs, $account)
    {
        $priority = 1;
        foreach ($phoneCodecs as $codecKey => $codec) {
            $key = "account.$account.codec.$priority";
            $this->psetting("$key.enable", $codec['enable']);
            $this->psetting("$key.payload_type", $codecKey);
            $this->psetting("$key.priority", $codec['priority']);
            $this->psetting("$key.rtpmap", $codec['rtpmap']);
            $priority++;
        }
    }
}
