<?php
class DefaultController extends Controller
{
    /**
    * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
    * using two-column layout. See 'protected/views/layouts/column2.php'.
    */
    public $layout='//layouts/column2';

    /**
    * @return array action filters
    */
    public function filters()
    {
        return array(
                'accessControl', // perform access control for CRUD operations
        );
    }

    /**
    * Specifies the access control rules.
    * This method is used by the 'accessControl' filter.
    * @return array access control rules
    */
    public function accessRules()
    {
        return array(
            array('allow',  // allow all users to perform 'index' and 'view' actions
                'actions'=>array('index'),
            	'expression'=>AccessManager::checkSessionPermission($this->module->name),
            		
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions'=>array('create','update'),
            	'expression'=>AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions'=>array('delete'),
            	'expression'=>AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny',  // deny all users
                'users'=>array('*'),
            ),
        );
    }

    /**
    * Creates a new model.
    * If creation is successful, the browser will be redirected to the 'view' page.
    */
    public function actionCreate()
    {
        $model = new ExternalAgent();
        $this->performAjaxValidation($model);
        if (isset($_POST['ExternalAgent'])) {
            $model->attributes = $_POST['ExternalAgent'];
            $model->save();
        } 
        $this->renderPartialWithContextHelp('_form',array('model'=>$model),false,true);
    }

    /**
    * Updates a particular model.
    * If update is successful, the browser will be redirected to the 'view' page.
    * @param integer $id the ID of the model to be updated
    */
    public function actionUpdate($id) {
        
        $model = $this->loadModel($id);
	$this->performAjaxValidation($model);
        if (isset($_POST['ExternalAgent'])) {
            $model->attributes = $_POST['ExternalAgent'];
            $model->save();
        } 
        $this->renderPartialWithContextHelp('_form',array('model'=>$model),false,true);
    }

    /**
    * Deletes a particular model.
    * If deletion is successful, the browser will be redirected to the 'admin' page.
    * @param integer $id the ID of the model to be deleted
    */
    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            // we only allow deletion via POST request
            $this->loadModel($id)->delete();
            // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('index'));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    /**
    * Lists all models.
    */
    public function actionIndex()
    {
        $model = new ExternalAgent('search');
        $model->unsetAttributes();

        if (isset($_GET['ExternalAgent'])) {
            $model->attributes = $_GET['ExternalAgent'];
        }
        $this->render('index', array(
            'model' => $model,
            'enableAddButton' => ExternalAgent::allowExternalAgentCreate()
        ));
    }

    /**
    * Returns the data model based on the primary key given in the GET variable.
    * If the data model is not found, an HTTP exception will be raised.
    * @param integer the ID of the model to be loaded
    */
    public function loadModel($id) {

        $model= ExternalAgent::model()->findByPk($id);
        if($model===null) {
            throw new CHttpException(404,'The requested page does not exist.');
        }
        return $model;
    }

    /**
    * Performs the AJAX validation.
    * @param CModel the model to be validated
    */
    protected function performAjaxValidation($model)
    {
        if(isset($_POST['ajax']) && $_POST['ajax'] === 'external-agent-form') {

            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
    
    public function renderState($data,$row, $column) {
      
      $this->renderPartial('renderState',array('data'=>$data,'row'=>$row,'column'=>$column));
    }
    
    public function renderType($data,$row,$column) {
      return ExternalAgentTypes::readable($data->type);
    }
}
