<?php
class ExternalHostTypes {
    
    const roleForeign           = 0;
    const roleMaster            = 1;
    //const roleSlave             = 2;
    
    const typeStatic            = 3;
    const typeRegisteredInbound  = 4;
    const typeRegisteredOutbound = 5;
    
    public static function getHumanReadableTypeName($enum) {
        $typeName = 'undefined';
        switch ($enum) {
            case ExternalHostTypes::roleForeign: 
                $typeName = 'EXTERNAL_HOST_TYPE_ROLE_FOREIGN';
            break;
            case ExternalHostTypes::roleMaster : 
                $typeName = 'EXTERNAL_HOST_TYPE_ROLE_MASTER';
            break;
            /*case ExternalHostTypes::roleSlave:
                $typeName = 'EXTERNAL_HOST_TYPE_ROLE_SLAVE';
            break;*/
            
            case ExternalHostTypes::typeStatic: 
                $typeName = 'EXTERNAL_HOST_TYPE_TYPE_STATIC';
            break;
            case ExternalHostTypes::typeRegisteredInbound: 
                $typeName = 'EXTERNAL_HOST_TYPE_TYPE_REGISTERED_INBOUND';
            break;
            case ExternalHostTypes::typeRegisteredOutbound:
                $typeName = 'EXTERNAL_HOST_TYPE_TYPE_REGISTERED_OUTBOUND';
            break;
        }
        return __($typeName);
    }
}


class ExternalHost extends PhonalisaRecord {

    //public  $id;
    public  $active = 1;
    public  $name;
    public  $role;
    public  $prefix;
    public  $our_prefix;
    public  $host;
    public  $type;
    public  $username;
    public  $password;
    
    // client specific
    public  $register_state;
    public  $device_name;
    public  $last_contact;
    public  $expires;
    public  $current_ip;
    
    public $tcp_transport;
    public $allowed_codecs;

    public $state;
    
    public $scriptExtension;
    
    public static function myAttributes() {
    
        return array(
            'active'=>'active',
            'name'=>'name',
            'role'=>'role',
            'prefix'=>'prefix',
        	'our_prefix' => 'our_prefix',
            'type'=>'type',
            'host'=>'host',
            'username'=>'username',
            'password'=>'password',
        	'register_state' => 'register_state',
        	'device_name' => 'device_name',
        	'last_contact' => 'last_contact',
        	'expires' => 'expires',
        	'current_ip' => 'current_ip',
        	 
        	'tcp_transport'=>'tcp_transport',
            'allowed_codecs'=>'allowed_codecs',
        	 
            'state'=>'state',
            'scriptExtension' => 'scriptExtension'
        );
    }
    
    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Calendar the static model class
     */
    public static function model($className=__CLASS__)
    {
        return parent::model($className);
    }
    
    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return strtolower('ExternalHost');
    }
    
    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
                array('prefix,name,username,password','required'),

                array('username','ext.PhonalisaValidators.uniqueAuthName'),
                array('host','requiredOnOutboundHost'),
                array('name,allowed_codecs','length', 'max'=>255),
                array('prefix','numerical','integerOnly'=>true),
                array('name','match','pattern'=>'/^[0-9a-zA-Z[\]\/\-{}()]+$/'),
                array('name', 'ext.PhonalisaValidators.isUnique'),
                array('host','length', 'max'=>255),
        		array('our_prefix','length', 'max'=>10),
                array('state','safe'), // FIXME 
                array('role, type' , 'numerical', 'integerOnly'=>true),
                array('active, tcp_transport','boolean'),
                array('username, password, scriptExtension','safe'),
        );
    }
    
    public function requiredOnOutboundHost($attribute,$params) {

        if ($this->type == ExternalHostTypes::typeRegisteredOutbound || $this->type == ExternalHostTypes::typeStatic) {

            $requiredValidator = new CRequiredValidator();
            $requiredValidator->attributes = array($attribute);
            if ( !$requiredValidator->validate($this)) {
                return FALSE;
            }
        }
        return TRUE;
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
        );
    }
    
    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {

        return array(
            'active'=>__('EXTERNAL_HOST_MODEL_ACTIVE'),
            'name'=>__('EXTERNAL_HOST_MODEL_NAME'),
            'role'=>__('EXTERNAL_HOST_MODEL_ROLE'),
            'prefix'=>__('EXTERNAL_HOST_MODEL_PREFIX'),
        	'our_prefix' =>__('EXTERNAL_HOST_MODEL_OUR_PREFIX'),
            'host'=>__('EXTERNAL_HOST_MODEL_HOST'),
            'type'=>__('EXTERNAL_HOST_MODEL_TYPE'),
            'username'=>__('EXTERNAL_HOST_MODEL_USERNAME'),
            'password'=>__('EXTERNAL_HOST_MODEL_PASSWORD'),
            'tcp_transport'=>__('EXTERNAL_HOST_MODEL_TCP_TRANSPORT'),
            'allowed_codecs'=>__('EXTERNAL_HOST_MODEL_ALLOWED_CODECS'),
            "register_state" => __("EXTERNAL_HOST_MODEL_REGISTER_STATE"),
            'scriptExtension' => __('USER_MODEL_SCRIPT_EXTENSION')
            
        );
    }
    
    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search($order = 'name')
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.
        
        $criteria=new CDbCriteria;
        $filter = $this->getFilter($criteria, $order);
        
        $filter['pagination'] = array(
            'pageSize'=> getGridPageSize('external-host-grid'),
        );
        
        if ($filter != null) {
            return new CPhonalisaDataProvider($this, array(
                    'criteria'=>$criteria,
            ));
        }
    }
}