<?php

class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'create', 'update', 'delete'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function actionIndex()
    {
        $model = new IVR('search');
        $model->unsetAttributes();  // clear any default values
        if (isset($_GET['IVR'])) {
            $model->attributes = $_GET['IVR'];
        }
        $this->render('index', array(
            'model' => $model,
        ));
    }

    /**
     * Creates a new model.
     * If creation is successful, the browser will be redirected to the 'index' page.
     */
    public function actionCreate()
    {
        $model = new IVR;
        $availableKeys = $model->getAvailableKeys();
        foreach ($availableKeys as $keyName) {
            $meta = 'key_' . $keyName . '_meta';
            if ($model->$meta == '') {
                $model->$meta = '---';
            }
        }

        $this->performAjaxValidation($model);

        if (isset($_POST['IVR'])) {
            $model->attributes = $_POST['IVR'];
            foreach ($model->getAvailableKeys() as $key) {
                $keyPost = $_POST['IVR']['keys'][$key];
                $actionAttributeName = 'key_' . $key . '_action';
                $targetAttributeName = 'key_' . $key . '_target';
                $targetTypeAttributeName = 'key_' . $key . '_targetType';
                $targetExtensionAttributeName = 'key_' . $key . '_targetExtension';
                $voiceMailAttributeName = 'key_' . $key . '_meta';

                $model->$actionAttributeName = $keyPost['action'];
                $model->$voiceMailAttributeName = $keyPost['meta'];

                if (isset($keyPost['target'][$model->$actionAttributeName])) {
                    $targetTypeParts = explode('_', $keyPost['target'][$model->$actionAttributeName]);

                    $lastIndex = count($targetTypeParts) - 1;

                    $model->$targetAttributeName = isset($targetTypeParts[$lastIndex]) ? $targetTypeParts[$lastIndex] : null;
                    $model->$targetExtensionAttributeName = isset($targetTypeParts[1]) ? base64_decode($targetTypeParts[1]) : null;
                    $model->$targetTypeAttributeName = is_numeric($targetTypeParts[0]) ? $targetTypeParts[0] : null;

                    if ($model->$targetTypeAttributeName == TargetType::Manual) {
                        $model->$targetAttributeName = $keyPost['target']['customNumber'];
                    }
                }
            }
            if ($model->save()) {
                $this->redirect(array('index', 'id' => $model->id));
            }
        }
        $this->render('_form', array('model' => $model));
    }

    /**
     * Updates a particular model.
     * If update is successful, the browser will be redirected to the 'index' page.
     * @param integer $id the ID of the model to be updated
     */
    public function actionUpdate($id)
    {
        $model = $this->loadModel($id);

        //Uncomment the following line if AJAX validation is needed
        //$this->performAjaxValidation($model);

        if (isset($_POST['IVR'])) {
            $model->attributes = $_POST['IVR'];
            foreach ($model->getAvailableKeys() as $key) {

                $keyPost = $_POST['IVR']['keys'][$key];

                $actionAttributeName = 'key_' . $key . '_action';
                $targetAttributeName = 'key_' . $key . '_target';
                $targetTypeAttributeName = 'key_' . $key . '_targetType';
                $targetExtensionAttributeName = 'key_' . $key . '_targetExtension';
                $voiceMailAttributeName = 'key_' . $key . '_meta';

                $model->$actionAttributeName = $keyPost['action'];
                $model->$voiceMailAttributeName = $keyPost['meta'];

                if (isset($keyPost['target'][$model->$actionAttributeName])) {
                    $targetTypeParts = explode('_', $keyPost['target'][$model->$actionAttributeName]);

                    $lastIndex = count($targetTypeParts) - 1;

                    $model->$targetAttributeName = isset($targetTypeParts[$lastIndex]) ? $targetTypeParts[$lastIndex] : null;
                    $model->$targetExtensionAttributeName = isset($targetTypeParts[1]) ? base64_decode($targetTypeParts[1]) : null;
                    $model->$targetTypeAttributeName = is_numeric($targetTypeParts[0]) ? $targetTypeParts[0] : null;

                    if ($model->$targetTypeAttributeName == TargetType::Manual) {
                        $model->$targetAttributeName = $keyPost['target']['customNumber'];
                    }
                }

                if ($model->$actionAttributeName == 3) {
                    $model->$targetTypeAttributeName = TargetType::VoiceMail;
                }

                if ($model->$actionAttributeName == '0' || $model->$actionAttributeName == '') {
                    $model->$actionAttributeName = '';
                    $model->$targetAttributeName = '';
                    $model->$targetTypeAttributeName = '';
                    $model->$targetExtensionAttributeName = '';
                    $model->$voiceMailAttributeName = '';
                }
            }
            if ($model->save()) {
                $this->redirect(array('index'));
            }
        }
        $this->render('_form', array('model' => $model));
    }

    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'index' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            // we only allow deletion via POST request
            $this->loadModel($id)->delete();
            // if AJAX request (triggered by deletion via index grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('index'));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = IVR::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'ivr-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
}
