<?php

class DefaultController extends Controller
{

    const EMPTY_RULE = '---';

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';
    private $dateTimeProfileCache = null;
    private $availableRoutingTargetsCache = null;

    private function getDateTimeProfiles()
    {
        if ($this->dateTimeProfileCache === null) {
            $this->dateTimeProfileCache = DateTimeProfile::model()->find(array(), 'title');
        }
        return $this->dateTimeProfileCache;
    }

    private function getAvailableRoutingTargets($model = null)
    {
        if ($this->availableRoutingTargetsCache === null) {
            if ($model) {
                $this->availableRoutingTargetsCache = $model->getAvailableRoutingTargets();
            } else {
                $this->availableRoutingTargetsCache = RoutingInbound::model()->getAvailableRoutingTargets();
            }
        }
        return $this->availableRoutingTargetsCache;
    }

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'admin', 'delete', 'ajaxValidate', 'select', 'export'),
                //'users'=>array('@'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    public function actionSelect()
    {
        if (isset($_POST['gw_id'])) {
            $this->redirect(array('admin', 'gw_id' => $_POST['gw_id']));
        } else {
            // Redirect user gateway creating page if no gateways exists
            $this->redirect(array('/routingGateways/'));
        }
    }

    public function actionIndex()
    {
        $models = Gateway::model()->findAll();
        if (count($models) > 0) {
            $gatewayModel = $this->getGatewayModel();
            $this->redirect(array('admin', 'gw_id' => $gatewayModel->id));
        }
        $models = CHtml::listData($models, "id", "name");
        $this->render('create', array(
            'models' => $models,
        ));
    }

    private function getGatewayModel()
    {
        $models = Gateway::model()->find(array('active' => "1"), "name", 1, 0);
        if (empty($models)) {
            $models = Gateway::model()->find(array(), "name", 1, 0);
        }
        return $models[0];
    }

    public function actionAdmin()
    {
        $model = new RoutingInbound('search');
        if (isset($_POST['RoutingInbound'])) {
            foreach ($_POST['RoutingInbound'] as $id => $post) {
                $this->updateRoutingRules($id, $post);
            }
        }
        $this->render('admin', array('model' => $model));
    }

    public function updateRoutingRules($id, $post)
    {
        $model = RoutingInbound::model()->findByPk($id);
        if (!$model) {
            $model = new RoutingInbound();
        }
        $model->attributes = $this->getAttributeValues($id, $post);
        if ($model->targetType == TargetType::CentralDropTarget || $model->targetType == TargetType::CallThrough) {
            $model->scenario = 'centralDropTarget';
        }
        $model->save();
    }

    public function getAttributeValues($id, $attributes)
    {
        if ($attributes['pattern'] === self::EMPTY_RULE) {
            $attributes['pattern'] = '';
            $attributes['customPattern'] = '';
            $attributes['patternType'] = '';
        }
        if ($attributes['target'] === self::EMPTY_RULE) {
            $attributes['target'] = '';
            $attributes['customNumber'] = '';
            $attributes['targetType'] = '';
        }

        if (!$this->getTargetValues($attributes)) {
            $targetType = $attributes['target'];
            $attributes['target'] = $attributes['customNumber'];
            if ($targetType == TargetType::Extension) {
                $attributes['target'] = RoutingInbound::TARGET_PATTERN_EXTENSION;
            }

            if ($targetType == TargetType::CentralDropTarget) {
                $attributes['target'] = '';
            }
            $attributes['targetType'] = $targetType;
        }

        $pattern = $attributes['pattern'];
        $attributes['pattern'] = $attributes['customPattern'];
        $attributes['patternType'] = PatternType::Manual;
        if ($pattern === (string) PatternType::DID) {
            $attributes['pattern'] = RoutingInbound::TARGET_PATTERN_ALL_NUMBERS;
            $attributes['patternType'] = PatternType::DID;
        } else if (startsWith($pattern, (string) PatternType::ExtensionRange)) {
            $extension = explode('_', $pattern);
            $attributes['pattern'] = $extension[1];
            $attributes['patternType'] = PatternType::ExtensionRange;
        }

        if ($attributes['sort'] == 'new') {
            $attributes['sort'] = RoutingInbound::model()->countByAttributes(array('gw_id' => $_GET['gw_id']));
        }
        return $attributes;
    }

    public function getTargetValues(&$postedTarget)
    {
        $targets = explode('_', $postedTarget['target']);
        if (isset($targets[1])) {
            $postedTarget['targetType'] = $targets[0];
            $postedTarget['target'] = $targets[1];
        } else {
            return false;
        }
        return $postedTarget;
    }

    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            // we only allow deletion via POST request
            $model = RoutingInbound::model()->findByPk($id);
            $model->delete();
            // if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array(''));
            }
            Yii::app()->end();
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    public function actionAjaxValidate()
    {
        $model = new RoutingInbound();
        $model->gw_id = 'validate';
        $errors = array();
        foreach ($_REQUEST['RoutingInbound'] as $modelId => $attributes) {
            $attributes = $this->getFormatedAttribute($attributes, 'pattern', 'customPattern');
            $attributes = $this->getFormatedAttribute($attributes, 'target', 'customNumber');

            $model->attributes = $attributes;
            $sort = $attributes['sort'];
            if ($modelId == 'new') {
                $model->sort = 0;
            }
            $validateOutput = CActiveForm::validate($model);
            $decodetOutput = (array) json_decode($validateOutput);
            foreach ($decodetOutput as $key => $value) {
                //overwrite inputField Id
                $newKey = '';
                foreach (explode('_', $key) as $index => $part) {
                    if ($index == 0) {
                        $newKey .= $part . '_' . $modelId;
                    } else {
                        $newKey.='_' . $part;
                    }
                }
                $errors[$modelId][$newKey] = $value;
            }
        }
        $errors = $this->removeNewAttributesFromErrorList($errors, 'RoutingInbound_new_pattern');
        $errors = $this->removeNewAttributesFromErrorList($errors, 'RoutingInbound_new_target');
        if (!empty($errors)) {
            echo json_encode($errors);
        }
        Yii::app()->end();
    }

    private function getFormatedAttribute($attributes, $attributeName, $customAttributeName)
    {
        $searchForType = (string) PatternType::Manual;
        if ($attributeName === 'target') {
            $searchForType = (string) TargetType::Manual;
        }
        if ($attributes[$attributeName] === self::EMPTY_RULE) {
            $attributes[$attributeName] = '';
        }
        if ($attributes[$attributeName] == $searchForType) {
            $attributes[$attributeName] = $attributes[$customAttributeName];
        }
        return $attributes;
    }

    private function removeNewAttributesFromErrorList($errors, $attributeName)
    {
        if (isset($errors['new'][$attributeName])) {
            unset($errors['new'][$attributeName]);
            if (empty($errors['new'])) {
                unset($errors['new']);
            }
        }
        return $errors;
    }

    public function renderDescCell($data, $row, $cell)
    {
        $this->renderPartial("desc_input", array("model" => $data));
    }

    public function renderAktivCell($data)
    {
        $this->renderPartial("activeCell", array("data" => $data));
    }

    public function renderPatternCell($data, $row, $cell)
    {
        $listData = $this->getPatternListData($data);
        if (!isset($data->patternType) || $data->patternType === '') {
            $data->patternType = (string) PatternType::Manual;
        }
        if ($data->id === 'new' && !isset($data->patternType)) {
            $data->patternType = self::EMPTY_RULE;
        }

        $selectedPattern = $this->getSelectedRoutingPattern($data);
        $visbleStyle = $this->getPatternCellVisibleStyle($data);
        $name = 'RoutingInbound[' . $data->id . '][pattern]';
        $this->renderPartial("patternCell", array(
            "data" => $data,
            'listData' => $listData,
            'visbleStyle' => $visbleStyle,
            'name' => $name,
            "selectedPattern" => $selectedPattern
        ));
    }

    protected function getPatternListData($data)
    {
        $gateway = Gateway::model()->findByPk($data->gw_id);
        $listData = array(self::EMPTY_RULE => __('ROUTING_INBOUND_SELECT_PATTERN'));

        $didRange = (int) $gateway->did_to - (int) $gateway->did_from;
        if ($didRange <= 20) {
            for ($i = (int) $gateway->did_from; $i <= (int) $gateway->did_to; $i++) {
                $listData[PatternType::ExtensionRange . "_$i"] = $i;
            }
        }

        $listData[PatternType::Manual] = __('RENDER_TARGET_TYPE_MANUAL');
        $listData[PatternType::DID] = __('ROUTING_INBOUND_DID') . " " . RoutingInbound::TARGET_PATTERN_ALL_NUMBERS;
        return $listData;
    }

    protected function getSelectedRoutingPattern($data)
    {
        $selectedTargetNumber = self::EMPTY_RULE;
        if ($data->patternType == (string) PatternType::ExtensionRange) {
            $selectedTargetNumber = $data->patternType . '_';
            $selectedTargetNumber .= $data->pattern;
        }
        if ($data->patternType == (string) PatternType::DID) {
            $selectedTargetNumber = PatternType::DID;
        }
        if ($data->patternType == TargetType::Manual) {
            $selectedTargetNumber = TargetType::Manual;
            if ($data->pattern == RoutingInbound::TARGET_PATTERN_ALL_NUMBERS) {
                $data->patternType = PatternType::DID;
                $selectedTargetNumber = PatternType::DID;
            }
        }
        return $selectedTargetNumber;
    }

    protected function getPatternCellVisibleStyle($data)
    {
        $visbleStyle = '';
        if ($data->patternType != PatternType::Manual) {
            $visbleStyle = 'style="display:inline-block;"';
        }
        return $visbleStyle;
    }

    public function renderTargetCell($data)
    {
        $listData = $this->getTargetListData($data);
        $selectedTargetNumber = $this->getSelectedRoutingTarget($data);
        $visbleStyle = $this->getRoutingTargetCellVisibleStyle($data);
        $name = 'RoutingInbound[' . $data->id . '][target]';
        $this->renderPartial("targetCell", array(
            "name" => $name,
            "selectedTargetNumber" => $selectedTargetNumber,
            "listData" => $listData,
            "data" => $data,
            "visbleStyle" => $visbleStyle
        ));
    }

    protected function getTargetListData($data)
    {
        $listData = $this->getAvailableRoutingTargets($data);
        $listData[__('ROUTINGINBOUND_MODEL_TARGET_OTHER')] = array(
            TargetType::Extension => __('ROUTING_INBOUND_PATTERN_TO_EXTENSION') . " " . RoutingInbound::TARGET_PATTERN_EXTENSION,
            TargetType::CentralDropTarget => __('SYSTEM_GLOBAL_CONFIG_KEY_DP_DROPTARGET_SHORT'),
            TargetType::CallThrough => __('ROUTINGINBOUND_MODEL_TARGET_OTHER_CALL_TROUGH'),
            TargetType::Manual => __('RENDER_TARGET_TYPE_MANUAL')
        );
        return $listData;
    }

    protected function getSelectedRoutingTarget($data)
    {
        $selectedTargetNumber = TargetType::Manual;
        if ($data->targetType != TargetType::Manual) {
            $selectedTargetNumber = $data->targetType . '_';
            $selectedTargetNumber .= $data->target;
        }
        if ($data->targetType == TargetType::CallThrough) {
            $selectedTargetNumber = TargetType::CallThrough;
        }
        if ($data->targetType == TargetType::Manual && $data->target == RoutingInbound::TARGET_PATTERN_EXTENSION) {
            $data->targetType = TargetType::Extension;
            $selectedTargetNumber = TargetType::Extension;
        }
        if ($data->targetType == TargetType::Extension) {
            $selectedTargetNumber = TargetType::Extension;
        }
        if ($data->targetType == TargetType::CentralDropTarget) {
            $selectedTargetNumber = TargetType::CentralDropTarget;
        }

        return $selectedTargetNumber;
    }

    protected function getRoutingTargetCellVisibleStyle($data)
    {
        $visbleStyle = '';
        if ($data->targetType != TargetType::Manual) {
            $visbleStyle = 'style="display:inline-block;"';
        }
        return $visbleStyle;
    }

    public function renderDateTimeProfile(PhonalisaRecord $data, $row, $column)
    {
        $listData = array('' => self::EMPTY_RULE);
        $listData = array_merge($listData, CHtml::listData($this->getDateTimeProfiles(), 'id', 'title'));
        $this->renderPartial('dateTimeProfileCell', array('listData' => $listData, 'data' => $data));
    }

    public function renderHiddenSortCell($data)
    {
        $this->renderPartial("hiddenSortCell", array("data" => $data));
    }

    public function renderDialoutPrefixCell($data)
    {
        if ($data->overwriteDialPrefix != '') {
            $checked = true;
            $displayTextField = 'display:inline;';
        } else {
            $checked = false;
            $displayTextField = 'display:none;';
        }
        $this->renderPartial("dialoutPrefixCell", array(
            "data" => $data,
            "checked" => $checked,
            "displayTextField" => $displayTextField
        ));
    }

    public function renderCidPrefixCell($data)
    {
        $this->renderPartial("cidPrefixCell", array("data" => $data));
    }

    public function renderScoreDropdown(PhonalisaRecord $data, $row, $column)
    {
        $listData = array(self::EMPTY_RULE);
        for ($i = 1; $i <= 9; $i++) {
            $text = __('ROUTING_INBOUND_SCORE_LEVEL_NEUTRAL');
            if ($i < 5) {
                $text = __('ROUTING_INBOUND_SCORE_LEVEL_REPUTABLE');
            } else if ($i > 5) {
                $text = __('ROUTING_INBOUND_SCORE_LEVEL_DUBIOUS');
            }
            $listData[$text][$i] = $i;
        }
        $this->renderPartial('scoreCell', array('listData' => $listData, 'data' => $data));
    }

    public function actionExport($gw_id = '')
    {
        if ($gw_id) {
            ExportHelper::exportSpecificModelAsCSV('RoutingInbound', array('gw_id' => $gw_id));
        } else {
            ExportHelper::exportModelAsCSV('RoutingInbound');
        }
    }
}
