<?php

class RoutingInbound extends PhonalisaRecord
{
    const TARGET_PATTERN_EXTENSION = '$1';
    const TARGET_PATTERN_ALL_NUMBERS = '(.*)';

    public $id;
    public $target;         //Target ID
    public $targetType;     //ExtensionType
    public $pattern;
    public $patternType;
    public $gw_id;
    public $sort;
    public $overwriteDialPrefix;
    public $active = 1;
    public $dateTimeProfile;
    public $cid_prefix;
    public $desc;
    public $score = 0;

    public static function myAttributes()
    {
        return array(
            'active' => 'active',
            'overwriteDialPrefix' => 'overwriteDialPrefix',
            'target' => 'target',
            'targetType' => 'targetType',
            'pattern' => 'pattern',
            'patternType' => 'patternType',
            'gw_id' => 'gw_id',
            'sort' => 'sort',
            'dateTimeProfile' => 'dateTimeProfile',
            'cid_prefix' => 'cid_prefix',
            'desc' => 'desc',
            'score' => 'score'
        );
    }

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return RoutingInbound the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'routingInbound';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('pattern, gw_id, sort', 'required'),
            array('target', 'required', 'except' => 'centralDropTarget'),
            array('sort, score', 'numerical', 'integerOnly' => true),
            array('active, target, dateTimeProfile', 'safe'),
            array('overwriteDialPrefix', 'length', 'max' => 2),
            array('targetType, patternType', 'length', 'max' => 36),
            array('desc', 'length', 'max' => 255),
            array('cid_prefix', 'length', 'max' => 6),
            array('cid_prefix', 'match', 'pattern' => '/^[0-9a-zA-Z_-]*$/'),
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => __('ROUTING_INBOUND_MODEL_ID'),
            'target' => __('ROUTING_INBOUND_MODEL_TARGET'),
            'pattern' => __('ROUTING_INBOUND_MODEL_PATTERN'),
            'gw_id' => __('ROUTING_INBOUND_MODEL_GW_ID'),
            'sort' => __('ROUTING_INBOUND_MODEL_SORT'),
            'dateTimeProfile' => __('ROUTING_INBOUND_MODEL_DATE_TIME_PROFILE'),
            'overwriteDialPrefix' => __('ROUTING_INBOUND_MODEL_OVERWRITE_DIAL_PREFIX'),
            'active' => __('ROUTING_INBOUND_MODEL_ACTIVE'),
            'cid_prefix' => __('ROUTING_INBOUND_MODEL_CID_PREFIX'),
            'desc' => __('ROUTING_INBOUND_MODEL_DESC'),
            'score' => __('ROUTING_INBOUND_MODEL_SCORE')
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CPhonalisaDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.
        $arrayModels = array();
        $newmodel = new RoutingInbound();
        if (isset($_GET['gw_id'])) {
            $newmodel->gw_id = $_GET['gw_id'];
        }

        $arrayModels = RoutingInbound::model()->find(array('gw_id' => $newmodel->gw_id), 'sort');
        $newmodel->id = "new";
        $newmodel->sort = "new";
        $arrayModelsCount = count($arrayModels);
        if ($arrayModelsCount >= 1) {
            $lastModel = $arrayModels[$arrayModelsCount - 1];
            $newmodel->patternType = $lastModel->patternType;
            if ($newmodel->patternType === (string) PatternType::ExtensionRange) {
                $newmodel->pattern = $lastModel->pattern;
            }
        }
        $arrayModels[] = $newmodel;

        $dataProvider = new CArrayDataProvider($arrayModels, array(
            'id' => 'key',
            'sort' => array('defaultOrder' => 'sort ASC'),
            'pagination' => false,
        ));
        return $dataProvider;
    }

    public function getCheckBoxDiv($colse = false)
    {
        $checkBoxDiv = '';
        if (!$colse) {
            $checkBoxDiv = '<div style="float:left;padding:5px 10px 0px 5px">';
        } else {
            $checkBoxDiv = '</div>';
        }
        return $checkBoxDiv;
    }

    public function getAvailableRoutingTargets()
    {
        // FIXME: order by extention
        $ivrs = IVR::model()->find(array(), 'extension');
        $listData = array();
        $listData['---'] = __('ROUTING_IVR_SELECT_TARGET');

        $users = User::model()->find(array(), 'extension');

        if (!empty($users)) {
            foreach ($users as $user) {
                $listData[__('RENDER_TARGET_USER')][(int) $user->extension][TargetType::User . '_' . $user->id] = getFormatedUsername($user);
            }
        }

        $groups = Groups::model()->find(array(), 'extension');
        $availableGroups = array();
        foreach ($groups as $group) {
            if ($group->extension != null) {
                $availableGroups[] = $group;
            }
        }
        if (!empty($availableGroups)) {
            foreach ($availableGroups as $group) {
                $listData[__('RENDER_TARGET_GROUP')][(int) $group->extension][TargetType::Group . '_' . $group->id] = '<' . $group->extension . '> ' . $group->name;
            }
        }

        if (!empty($ivrs)) {
            foreach ($ivrs as $ivr) {
                $listData[__('RENDER_TARGET_IVR')][(int) $ivr->extension][TargetType::Ivr . '_' . $ivr->id] = '<' . $ivr->extension . '> ' . $ivr->title;
            }
        }

        $callGroups = CallGroup::model()->find(array(), 'extension');
        if (!empty($callGroups)) {
            foreach ($callGroups as $callGroup) {
                $listData[__('CALL_GROUP')][(int) $callGroup->extension][TargetType::CallGroup . '_' . $callGroup->id] = '<' . $callGroup->extension . '> ' . $callGroup->name;
            }
        }

        $queues = Queue::model()->find(array(), 'extension');
        if (!empty($queues)) {
            foreach ($queues as $queue) {
                $listData[__('RENDER_TARGET_QUEUE')][(int) $queue->extension][TargetType::Queue . '_' . $queue->id] = '<' . $queue->extension . '> ' . $queue->name;
            }
        }

        $conferences = Conference::model()->find(array(), 'extension');
        if (!empty($conferences)) {
            foreach ($conferences as $conference) {
                $listData[__('RENDER_TARGET_CONFERENCE')][(int) $conference->extension][TargetType::Conference . '_' . $conference->id] = '<' . $conference->extension . '> ' . $conference->name;
            }
        }

        if (!empty($users) && PhonalisaLicenseManager::faxGUIEnabled()) {
            foreach ($users as $user) {
                $listData[__('ROUTINGINBOUND_MODEL_TARGET_USER_FAX')][(int) $user->extension][TargetType::UserFax . '_' . $user->id] = 'Fax: ' . getFormatedUsername($user);
            }
        }
//         
//          if (!empty($availableGroups)) {
//             foreach ($availableGroups as $group) {
//                 $listData[__('ROUTINGINBOUND_MODEL_TARGET_GROUP_FAX')][(int)$group->extension][TargetType::GroupFax.'_'.base64_encode($group->name).'_'.$group->id] = 'Fax: <'.$group->extension.'> '.$group->name;
//             }
//         }

        $sortedListData = array();
        foreach ($listData as $type => $data) {
            if (is_array($data)) {
                ksort($data);
                foreach ($data as $value => $label) {
                    foreach ($label as $key => $value) {
                        $sortedListData[$type][$key] = $value;
                    }
                }
            } else {
                $sortedListData[$type] = $data;
            }
        }
        $availableRoutingTargets = $sortedListData;
        return $sortedListData;
    }

    public function renderGatewayDropdown()
    {
        $gateways = Gateway::model()->findAll();
        if (count($gateways) == 1) {
            return NULL;
        }
        
        $keyActive = __('ROUTING_INBOUND_MODEL_ACTIVE');
        $keyDisabled = __('ROUTING_INBOUND_MODEL_DISABLED');
        
        $items['active'] = array('label' => $keyActive);
        $items = $this->getGateways($gateways, $items, "1", "✓ ");
        $items['disabled'] = array('label' => $keyDisabled);
        $items = $this->getGateways($gateways, $items, "0", "× ");
     
        return $items;
    }

    private function getGateways($gateways, $items, $active, $symbol)
    {
        foreach ($gateways as $gateway)  { 
            if ($gateway->active === $active) {
                $items[] = array(
                    'label' => $symbol. $gateway->name,
                    'url' => 'index.php?r=routingInbound/default/admin&gw_id=' . $gateway->id,
                    'active' => $gateway->id == $_REQUEST['gw_id'],
                );
            }
        }
        return $items;
    }

//    public static function csvRelevantAttributes($modelName)
//    {
//        return ImportHelper::getCsvRelevantModelAttributes($modelName);
//    }

    public function getExportNotRelevantAttributes()
    {
        return array(
            'gw_id',
            'dateTimeProfile',
            );
    }
}
