<?php

/**
  model class name  = CDR
  form-id  == cdr
 */
class DefaultController extends Controller
{

    /**
     * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
     * using two-column layout. See 'protected/views/layouts/column2.php'.
     */
    public $layout = '//layouts/column2';

    /**
     * @return array action filters
     */
    public function filters()
    {
        return array(
            'accessControl', // perform access control for CRUD operations
        );
    }

    /**
     * Specifies the access control rules.
     * This method is used by the 'accessControl' filter.
     * @return array access control rules
     */
    public function accessRules()
    {
        return array(
            array('allow', // allow all users to perform 'index' and 'view' actions
                'actions' => array('index', 'logMessage', 'cdrExport', 'automatedCdrExport'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions' => array('delete'),
                'expression' => AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny', // deny all users
                'users' => array('*'),
            ),
        );
    }

    /**
     * Lists all models.
     */
    public function actionIndex()
    {
        $model = new CDR('search');
        $model->unsetAttributes();  // clear any default values
        $searchFor = array();
        
        if (isset($_GET['CDR'])) {
            $model->attributes = $_GET['CDR'];
            $searchFor = array(
                'source' => @$_REQUEST['CDR']['source'],
                'target' => @$_REQUEST['CDR']['target'],
                'type' => @$_REQUEST['CDR']['type']
            );
        }

        $this->render('index', array(
            'model' => $model,
            'searchFor' => $searchFor
        ));
    }

    public function actionLogMessage($id, $page = null, $sort = null)
    {
        $model = $this->loadModel($id);

        $logArray = explode("\n", $model->logmessages);
        $logArray = str_replace(' GMT ', ' ', $logArray);
        $providerData = array();
        foreach ($logArray as $key => $logmessage) {
            $logData = explode('  ', $logmessage);
            $dateType = explode(' ', $logData[0]);

            $date = @$dateType[0] . ' ' . @$dateType[1] . ' ' . @$dateType[2] . ' ' . @$dateType[3] . ' ' . @$dateType[4];
            $type = @$dateType[5];
            $message = @$logData[1];

            $providerData[$key]['id'] = $key;
            $providerData[$key]['date'] = $date;
            $providerData[$key]['type'] = $type;
            $providerData[$key]['message'] = $message;
        }
        $dataProvider = new CArrayDataProvider($providerData, array('pagination' => array('pagesize' => 100)));

        $this->render('logMessageView', array('dataprovider' => $dataProvider, 'lastPage' => $page, 'sort' => $sort));
    }

    /**
     * Returns the data model based on the primary key given in the GET variable.
     * If the data model is not found, an HTTP exception will be raised.
     * @param integer the ID of the model to be loaded
     */
    public function loadModel($id)
    {
        $model = CDR::model()->findByPk($id);
        if ($model === null) {
            throw new CHttpException(404, 'The requested page does not exist.');
        }
        return $model;
    }

    public function actionCdrExport()
    {
        $searchFor = $this->getSearchFor(
                array(
                    'source' => @$_REQUEST['source'],
                    'target' => @$_REQUEST['target'],
                    'type' => @$_REQUEST['type']
                )
        );
        $limit = 1000;
        $offset = 0;
        $this->printHeaderForFileDownload();
        $date = $this->setDateTimeForExport();

        $foundItem = true;
        while ($items = CDR::model()->find($searchFor, 'start_timestamp DESC', $limit, $offset)) {
            $foundItem = $this->processItems($items, $date);
            if ($foundItem == false) {
                break;
            }
            $offset += $limit;
        }
    }

    protected function getSearchFor($searchForArray)
    {
        $searchFor = array();
        foreach ($searchForArray as $key => $value) {
            if (!empty($value) || $value == "0") {
                $searchFor[$key] = $value;
            }
        }
        return $searchFor;
    }

    protected function setDateTimeForExport()
    {
        $days = $this->setOneDayIfNotSet();

        $currentDateTime = PhonalisaDate::newServerDate();
        return date('Y-m-d\TG:i:s\Z', strtotime($currentDateTime . ' - ' . $days . ' days'));
    }

    protected function setOneDayIfNotSet()
    {
        $days = '1';
        if (isset($_REQUEST['days'])) {
            $days = $_REQUEST['days'];
        }
        return $days;
    }

    protected function printHeaderForFileDownload()
    {
        $attributeLabels = $this->getAttributeLabels();
        $delimiter = $this->getCsvDelimiter();
        $fileName = "phonalisa_CDR_" . date('Y-m-d_G-i-s') . ".csv";

        header('Content-type: text/csv');
        header("Content-disposition: attachment; filename=$fileName");
        header("Pragma: no-cache");
        header("Expires: 0");
        print(implode($delimiter, $attributeLabels));
    }

    protected function getAttributeLabels()
    {
        $attributes = $this->getCsvHeader();
        $model = new CDR;
        $attributeLabels = array();
        foreach ($attributes as $attribute) {
            $allAttributeLabels = $model->attributeLabels();
            $attributeLabels[] = $allAttributeLabels[$attribute];
        }
        return $attributeLabels;
    }

    protected function getCsvHeader()
    {
        return array('start_timestamp', 'source', 'target', 'type', 'duration', 'did', 'meta');
    }

    protected function getCsvDelimiter()
    {
        return ';';
    }

    protected function processItems($items, $date)
    {
        foreach ($items as $item) {
            if ($this->processSearchForItems($item, $date) == false) {
                return false;
            }
        }
        return true;
    }

    protected function processSearchForItems($item, $date)
    {
        if ($item->start_timestamp >= $date) {
            $this->processItemOutput($item);
            return true;
        } else {
            return false;
        }
    }

    protected function processItemOutput($item)
    {
        echo $this->downloadExportData($item);
    }

    protected function downloadExportData($item)
    {
        $exportData = "\n";
        $delimiter = $this->getCsvDelimiter();
        $csvHeader = $this->getCsvHeader();

        foreach ($csvHeader as $attribute) {
            $exportData .= $item->$attribute . $delimiter;
        }
        return $exportData;
    }

    public function actionAutomatedCdrExport()
    {
        Yii::import('application.commands.*');
        $command = new AutomaticCsvExportCommand('automaticCsvExport', 'Phonalisa');
        $command->run(array(
            '-date',
            @$_REQUEST['date'],
            '-delete',
            @$_REQUEST['delete'],
            '-download',
            @$_REQUEST['download'],
        ));
    }

    /**
     * Deletes a particular model.
     * If deletion is successful, the browser will be redirected to the 'admin' page.
     * @param integer $id the ID of the model to be deleted
     */
    public function actionDelete($id)
    {
        if (Yii::app()->request->isPostRequest) {
            //we only allow deletion via POST request
            $this->loadModel($id)->delete();
            //if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if (!isset($_GET['ajax'])) {
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('index'));
            }
        } else {
            throw new CHttpException(400, 'Invalid request. Please do not repeat this request again.');
        }
    }

    public function renderTarget($data)
    {
        return $data->forwarded_target ? $data->target . ' => ' . $data->forwarded_target : $data->target;
    }

    public function renderType($data)
    {
        return $this->renderPartial('typeError', array('data' => $data));
    }

    public function renderStartTimestamp($data)
    {
        $startTimestamp = str_replace(array('T', 'Z'), array(' ', ''), $data->start_timestamp);
        $datePattern = PhonalisaDate::userDatePattern();
        $timePattern = PhonalisaDate::userTimePattern(true);
        $pattern = $datePattern . " " . $timePattern;

        $startTime = PhonalisaDate::convertServerDateTo($pattern, $startTimestamp, true);

        return $startTime;
    }

    /**
     * Performs the AJAX validation.
     * @param CModel the model to be validated
     */
    protected function performAjaxValidation($model)
    {
        if (isset($_POST['ajax']) && $_POST['ajax'] === 'cdr-form') {
            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
}
