<?php

/**
 * This is the model class for table "MusicOnHold".
 *
 * The followings are the available columns in table 'MusicOnHold':
 * @property integer $id
 * @property string $name
 * @property string $type
 */
class MusicOnHold extends PhonalisaRecord
{

    public $id;
    public $name;
    public $descr;
 

    public static function myAttributes() {

        return array(
        	'id'=>'id',
            'name'=>'name',
            'descr'=>'descr',
        );
    }

	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return Gateway the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'MusicOnHold';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('name','required'),

            array( 'descr,', 'length', 'max'=>64),                          
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('id, name, descr', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'id' => __('MUSIC_ON_HOLD_MODEL_ID'),
			'name' => __('MUSIC_ON_HOLD_MODEL_NAME'),
            'descr'=>__('MUSIC_ON_HOLD_MODEL_DESCR'),
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CPhonalisaDataProvider the data provider that can return the models based on the search/filter conditions.
	 */        
	public function search($order = 'name')
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;
		$filter = $this->getFilter($criteria, $order);
		
		$filter['pagination'] = array(
            'pageSize'=> getGridPageSize('music-on-hold-grid'),
        );
		
		if ($filter != null) {
			return new CPhonalisaDataProvider($this, $filter);
		}
		/*
		$criteria->compare('id',$this->id);
		$criteria->compare('name',$this->name,true);
        $criteria->compare('descr',$this->descr,true);
		return new CPhonalisaDataProvider($this, array(
			'criteria'=>$criteria,
		));*/
	}
	
	public function beforeDelete() {
	  
	  $audioFiles = AudioFile::model()->findAllByAttributes(array('type'=>AudioFileType::musicOnHold,'playlistID'=>$this->id));
	  while (count($audioFiles)) {
	      foreach ($audioFiles as $file) {
		if ( !$file->delete() ) {
		    Log::e('phonalisa-webinterface','Could not delete AudioFile : "'.$file->id.'" in MusicOnHold::beforeDelete()');
		}
	      }
	      $audioFiles = AudioFile::model()->findAllByAttributes(array('type'=>AudioFileType::musicOnHold,'playlistID'=>$this->id));
	  }
	  return parent::beforeDelete();
	  
	}

}