<?php

/**
model class name  = NumberTranslation
form-id  == number-translation
*/

class DefaultController extends Controller
{
    /**
    * @var string the default layout for the views. Defaults to '//layouts/column2', meaning
    * using two-column layout. See 'protected/views/layouts/column2.php'.
    */
    public $layout='//layouts/column2';

    /**
    * @return array action filters
    */
    public function filters()
    {
        return array(
                'accessControl', // perform access control for CRUD operations
        );
    }

    /**
    * Specifies the access control rules.
    * This method is used by the 'accessControl' filter.
    * @return array access control rules
    */
    public function accessRules()
    {
        return array(
            array('allow',  // allow all users to perform 'index' and 'view' actions
                'actions'=>array('index','saveNewSortOrder'),
            	'expression'=>AccessManager::checkSessionPermission($this->module->name),
            		
            ),
            array('allow', // allow authenticated user to perform 'create' and 'update' actions
                'actions'=>array('create','update'),
            	'expression'=>AccessManager::checkSessionPermission($this->module->name),
            ),
            array('allow', // allow admin user to perform 'admin' and 'delete' actions
                'actions'=>array('delete'),
            	'expression'=>AccessManager::checkSessionPermission($this->module->name),
            ),
            array('deny',  // deny all users
                'users'=>array('*'),
            ),
        );
    }

    public function actionSaveNewSortOrder() {

	foreach ($_POST['data'] as $id => $sort) {
	    $model = NumberTranslation::model()->findByPk($id);
	    $model->sort = $sort;
	    if ( !$model->save()) {
	      $error = true;
	    } else {
	      $error = false;
	    }
	} 
	echo json_encode($error);
    }
       
    /**
    * Creates a new model.
    * If creation is successful, the browser will be redirected to the 'view' page.
    */
    public function actionCreate()
    {
        $model = new NumberTranslation();
	//FIXME !? 
        $model->sort = NumberTranslation::model()->count();
        
        $this->performAjaxValidation($model);
        if (isset($_POST['NumberTranslation'])) {

            $model->attributes = $_POST['NumberTranslation'];	
	    if (isset($_POST['sip_headers'])) {
	      $model->sip_headers = json_encode($_POST['sip_headers']);
	    } 
            $model->save();

        } 
        $this->renderPartial('_form',array('model'=>$model),false,true);
    }

    /**
    * Updates a particular model.
    * If update is successful, the browser will be redirected to the 'view' page.
    * @param integer $id the ID of the model to be updated
    */
    public function actionUpdate($id) {
        
        $model = $this->loadModel($id);
        $this->performAjaxValidation($model);
        if (isset($_POST['NumberTranslation'])) {
            $model->attributes = $_POST['NumberTranslation'];
	    if (isset($_POST['sip_headers'])) {
	      $model->sip_headers = json_encode($_POST['sip_headers']);
	    } else {
	      $model->sip_headers = null;
	    } 
            $model->save();
        } 
        $this->renderPartial('_form',array('model'=>$model),false,true);
    }

    /**
    * Deletes a particular model.
    * If deletion is successful, the browser will be redirected to the 'admin' page.
    * @param integer $id the ID of the model to be deleted
    */
    public function actionDelete($id)
    {
        if(Yii::app()->request->isPostRequest)
        {
//             we only allow deletion via POST request
            $this->loadModel($id)->delete();

//             if AJAX request (triggered by deletion via admin grid view), we should not redirect the browser
            if(!isset($_GET['ajax']))
                $this->redirect(isset($_POST['returnUrl']) ? $_POST['returnUrl'] : array('index'));
        }
        else
            throw new CHttpException(400,'Invalid request. Please do not repeat this request again.');
    }

    /**
    * Lists all models.
    */
    public function actionIndex()
    {
        $model = new NumberTranslation();
        
        $this->render('index',array(
            'model'=>$model,
        ));
    }

    /**
    * Returns the data model based on the primary key given in the GET variable.
    * If the data model is not found, an HTTP exception will be raised.
    * @param integer the ID of the model to be loaded
    */
    public function loadModel($id) {

        $model= NumberTranslation::model()->findByPk($id);
        if($model===null)
            throw new CHttpException(404,'The requested page does not exist.');
        return $model;
    }

    /**
    * Performs the AJAX validation.
    * @param CModel the model to be validated
    */
    protected function performAjaxValidation($model)
    {
        if(isset($_POST['ajax']) && $_POST['ajax'] === 'number-translation-form') {

            echo CActiveForm::validate($model);
            Yii::app()->end();
        }
    }
    
    public function getTabularFormTabs($form,$model) {
       $tabs = array();
        $count = 0;
        $translateKeys = array(
			  'generalData'=>__('NUMBERTRANSLATION_MODEL_TAB_MENU_GENERAL_DATA'),
  			  'sipHeader'=>__('NUMBERTRANSLATION_MODEL_TAB_MENU_SIP_HEADER'),
        		);
        foreach ($translateKeys as $view => $tabLabel) {
            $tabs[] = array(
                'active'=>$count++ === 0,
                'label'=>$tabLabel,
                'content'=>$this->renderPartial($view, array('form'=>$form, 'model'=>$model), true),
            );
        }
        return $tabs;
      
    }
    
        
    public function getSipHeaderDataprovider($model) {
      if (!$model->sip_headers) {
	$model->sip_headers = json_encode(
	  array(
	    array(
		'id'=>0,
		'header'=>'',
		'value'=>'',
		'new_value'=>''
	    ),
	));
      }
      $rawData = json_decode($model->sip_headers);
      return new CArrayDataProvider($rawData,array('keyField'=>'id'));
    }
    
    public function  renderSipHeaderInput($data,$row,$column) {
      $attribute = $column->name;
      if ($column->name == 'header') {
	$html = CHtml::hiddenField('sip_headers['.$data->id.'][id]',$row,array('class'=>'hidenIndex'));
	$html .= CHtml::textField('sip_headers['.$data->id.']['.$column->name.']',$data->$attribute,array('class'=>'span2'));
      }else {
	$html = CHtml::textField('sip_headers['.$data->id.']['.$column->name.']',$data->$attribute,array('class'=>'span2'));
      }
      
      return $html;
    }
    
    public function renderBool($data) {
      $html = CHtml::hiddenField($data->id,$data->sort,array('class'=>'hidenSort'));
      if ($data->active) {
	$html.= '<span class="icon-ok"></span>';
      } else {
	$html .= '<span class="icon-remove"></span>';
      }
      return $html;
    }
    
}
